/* Copyright (c) 2017-2018 Marvin Eagle <trailblazer77_2000@yahoo.com>
 * 
 *		This file is part of Chord Sequencer.
 * 
 * 		Chord Sequencer is a program to create and interactively play
 * 		series of chords in the live performance, controlling chord
 * 		change by pedal (or other similar device). It uses midi and
 * 		must be connected to an external synthesizer. 
 *
 *    Chord Sequencer is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Chord Sequencer is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *   along with Chord Sequencer.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * 		Author: Marvin Eagle <trailblazer77_2000@yahoo.com>
 * 
*/

#include "examplewindow.h"
#include <iostream>
#include <stdio.h>
#include "note.h"
#include <glib.h>
#include <cstdio>
#include <fstream>
#include <cstdlib>
#include "RtMidi.h"
#include "myarea.h"
#include <config.h>
#include "gettext.h"
#include <gtk/gtk.h>
#include "mybutton.h"
#include <gdk/gdkkeysyms.h>
#include <gdkmm/rgba.h>
#include <gtkmm/fixed.h>
#include <gtkmm/frame.h>
#include <glib/gprintf.h>
#include <gtkmm/hvscale.h>
#include <glibmm/refptr.h>
#include <glibmm/ustring.h>
#include <gtkmm/separator.h>
#include <gtkmm/liststore.h>
#include <gtkmm/adjustment.h>
#include "customradiobutton.h"
#include "customradiowidget.h"
#include <gtkmm/comboboxtext.h>
#include <gtkmm/scrolledwindow.h>

#define _(string) gettext (string)

// to install from shell: inside sequencer-1.0 directory run:
// # this creates the configure script
// $ autoreconf --verbose --install --force
// $ ./configure --with-jack
// (or if you want alsa $ ./configure --with-alsa)
// $  make 


// This program is best paired with zynaddsubfx synthesizer:
// to run program:
// open jack and start it.
// inside sequencer-1.0/Sequencer run
// ./sequencer

// To connect to zynddsubfx:
// zynaddsubfx -I jack -O jack -a
// connect Chord Sequencer to zynaddsubfx->midi_input (MIDI tab in jack)
// connect zynaddsubfx to system (Audio tab in jack)
// change midi input channel in zynaddsubfx to 2

// To connect additional instrument for playing along with the chords:
// connect it to the usb by MIO 1-in 1-out USB to MIDI cable
// Start second zynaddsubfx -I jack -O jack -a
// In Jack midi tab, expand zynaddsubfx in the right pane, and 
// connect system to midi_input under zynaddsubfx in the right pane
// In the Jack audio tab connect zynaddsubfx to system

// the whole project is 7064 lines in 17 files
// Author : Copyright (c) 2017-2018 Marvin Eagle (except for RtMidi library,
// which has it's own authors)
// Chord Sequencer Version  1.1 completed July 4, 2018

// This is an old way of compiling, do not use it:
// Old compile line:
// g++ -D__UNIX_JACK__ -o sequencer sequencer.cpp RtMidi.cpp customradiobutton.cpp  examplewindow.cpp  myarea.cpp  mybutton.cpp  mywidget.cpp  note.cpp  -ljack -lasound -lpthread `pkg-config gtkmm-3.0 --cflags --libs`
// End of the old way of compiling

// Platform-dependent sleep routines.
#if defined(__WINDOWS_MM__)
  #include <windows.h>
  #define SLEEP( milliseconds ) Sleep( (DWORD) milliseconds ) 
#else // Unix variants
  #include <unistd.h>
  #define SLEEP( milliseconds ) usleep( (unsigned long) (milliseconds * 1000.0) )
#endif

ExampleWindow::ExampleWindow(const Glib::RefPtr<Gtk::Application>& app)
: 	m_HBox(false, 0),
	main_VBox( Gtk::ORIENTATION_VERTICAL,0),
	m_Button_Quit(_("Quit") ),
	save_button(_("Save") ),
	// chord edit buttons
	new_chord_button(_("New chord") ),
	delete_chord_button(_("Delete chord") ),
	copy_chord_button(_("Copy chord") ),
	paste_chord_button(_("Paste chord") ),
	// chord play buttons
	next_chord_button(_("Next chord") ),
	previous_chord_button(_("Previous chord") ),
	play_button(_("Play") ),
	stop_button(_("Stop ") ),
	
	area(),
	fixed_board(),
	button_chords_HBox(),
	modal_button_HBox(),
	modal_button_VBox(),
	
	
	// button boxes which will toggled  hidden or visible
	chord_edit_button_box(),
	chord_play_button_box(), 
	note_edit_button_box(),
	sequence_button_box(),
	note_chord_play_radio_box(Gtk::Orientation::ORIENTATION_VERTICAL),
	
	// note edit buttons
	spacer_rad(),
	play_note_rad(),
	play_chord_rad(),
	dont_play_rad(),
	adj ( Gtk::Adjustment::create(0.0, 0.0, 128.0, 0.1, 1.0, 1.0) ),
	note_HBox(),
	instrument_HBox(),
	note_label(),
	instrument_label(),
	separator1(),
	separator2(),
	display_VBox(),
	normal_rad(),
	sharp_rad(),
	flat_rad(),
	sharpsharp_rad(),
	flatflat_rad(),
	scale_HBox(),
	set_velocity_b(_("Set velocity") ),
	message(),
	scroll(),
	scroll_VBox(),
	output(),
	current_sequence_name(),
	s_win(), 
	ks_window(),
	menu_box(),
	menubar(),
	manual(),
	radio_menu_items()

{
	
	midiout = 0; 
	api = RtMidi::UNSPECIFIED;
	// RtMidiOut constructor
	try
	{
		//midiout = new RtMidiOut(api);
		midiout = new RtMidiOut(api, "Chord Sequencer");
	}
	catch ( RtMidiError &error )
	{
		error.printMessage();
		exit( EXIT_FAILURE );
	}

	 // Call function to select port.
	try 
	{
		//chooseMidiPort(midiout);
		std::cout << "\n" << _("Opening virtual port") << "\n";
		midiout->openVirtualPort();
		
	}
	catch ( RtMidiError &error )
	{
		error.printMessage();
	}
	
	read_channel_file(); // sets the channel variable
	
	// Program change: 192, 5 = changes the instrument
	// Use Zynaddsubfx Ultra Soft instrument from the Collection bank
	message.push_back( 192 + channel - 1);
	message.push_back( 19 );
	midiout->sendMessage( &message );
	
	//  Quarter Frame message
	message[0] = 0xF1;
	message[1] = 60;
	midiout->sendMessage( &message );

	// Control Change: 176, 7, 100 (volume)
	message[0] = 176  + channel - 1;
	message[1] = 7;
	message.push_back( 100 );
	midiout->sendMessage( &message );
	
	// Poly on message
	message[0] = 176  + channel - 1;
	message[1] =  127;
	message[2] = 0;
	midiout->sendMessage( &message );
	
	// to turn off key repeat (which interprets holding down a key as a
	// multiple key presses) type at the terminal: xset r off
	int s = system("xset r off");
	
	mode = 1; // 1 for "Edit chord"
			//  2 for "Play chord"
			//  3 for "Edit note"
			//  4 for "Sequence"
	selected_chord_number = 1; // corresponds to the element # 0 in 
						// Glist area.current_sequence
	multiple_select = false;
	set_title("Chord Sequencer");
	playing = false;
	
	
	this->signal_key_press_event().connect(
		sigc::mem_fun(*this, &ExampleWindow::onKeyPress), false);
	this->signal_key_release_event().connect(
		sigc::mem_fun(*this, &ExampleWindow::onKeyRelease), false);
	
	
	for (int i=0; i<83; i++)
	{
		keys[i] = new MyWidget(); // index 0 will be ignored
	}
	for (int i=0; i<79; i++)
	{
		boxes[i] = new Gtk::EventBox(); // index 0 will be ignored
	}
	
	// GList selected_notes always has at least one note, 
	// which is not counted or displayed
	selected_note = NULL;
	selected_notes = new GList();
	dummy_selected_note = new Note(-1000);
	selected_notes->data = dummy_selected_note;
	copied_chord = NULL;
	sequence_names = NULL;
	is_sequence_named = false;
	
	
	/////////////// SETUP GUI ////////////////
	
	
	ks_window.shortcut_strings[0] =  new Glib::ustring(_("Play Chord") );
	ks_window.shortcut_strings[1] =  new Glib::ustring(_("Previous Chord") );
	ks_window.shortcut_strings[2] =  new Glib::ustring(_("Stop Chord") );
	ks_window.shortcut_strings[3] =  new Glib::ustring(_("New Chord") );
	ks_window.shortcut_strings[4] =  new Glib::ustring(_("Copy Chord") );
	ks_window.shortcut_strings[5] =  new Glib::ustring(_("Paste Chord") );
	
	ks_window.read_shortcuts_file();
	// ks_window.play_entry.set_text( *(new Glib::ustring("3") ) );
	
	mode_note_edit_button = new MyButton();
	mode_sequence_button = new MyButton();
	mode_chord_edit_button = new MyButton();
	mode_chord_play_button = new MyButton();
	mode_chord_edit_button->pressed = true; // start with "Edit chord" mode
	mult_select_b = new MyButton();
	
	
	box_note_edit = new Gtk::EventBox();
	box_sequence = new Gtk::EventBox();
	box_chord_edit = new Gtk::EventBox();
	box_chord_play = new Gtk::EventBox();
	
	box_mult_select = new Gtk::EventBox();
	
	// sc is a note velocity scale
	sc = new Gtk::HScale(adj);
	sc->set_draw_value(true);
	sc->set_sensitive(false);
	sc->set_digits(0);
	sc->set_value_pos(Gtk::POS_TOP);
	sc->set_size_request(180, -1);
		
	set_border_width(6);
	set_default_size(900, 200);
	set_size_request(900, 200);
	area_width = 3392;
	area.set_size_request (area_width, 522);
	scroll.set_size_request(848, 550);
	scroll.add(scroll_VBox); // this is container for area and button_chords_HBox
	scroll_VBox.pack_start(area, Gtk::PACK_SHRINK, 0);
	scroll_VBox.pack_start(button_chords_HBox, Gtk::PACK_SHRINK, 0);
	scroll_adj = scroll.get_hadjustment();
	
	add(main_VBox); // This is main Vertical Box. It has only two rows:
				// one is menu bar and second is fixed_board
				
	
	// setup menu			
	menu_box.set_orientation(Gtk::ORIENTATION_VERTICAL);
	application_menu_item = Gtk::manage(new Gtk::MenuItem(_("Application") ) ); 
	application_submenu = Gtk::manage(new Gtk::Menu);
	application_menu_item->set_submenu(*application_submenu); 
		
    shortcuts_subitem = Gtk::manage(new Gtk::MenuItem(_("Keyboard Shortcuts") ) );
    application_submenu->append(*shortcuts_subitem);
    
    channel_subitem = Gtk::manage(new Gtk::MenuItem(_("Midi Output Channel") ) );
    application_submenu->append(*channel_subitem);
    
    channel_submenu = Gtk::manage(new Gtk::Menu);
    channel_subitem->set_submenu(*channel_submenu);
    
    Gtk::RadioButton::Group group;
    Gtk::RadioMenuItem::Group gr = (Gtk::RadioMenuItem::Group) group;
   												
    
    for(int i = 0; i< 16; i++)
    {
		std::string s = std::to_string(i+1);
		char const * pchar = s.c_str();
		
		Glib::ustring ustr(pchar);
		radio_menu_items[i] = Gtk::manage(new Gtk::RadioMenuItem(gr, ustr, false));
		channel_submenu->append(*radio_menu_items[i] );
		if(channel - 1 == i)
			radio_menu_items[i]->set_active(true);
		
		radio_menu_items[i]->signal_activate().connect
					(sigc::bind<Gtk::RadioMenuItem*, int > (sigc::mem_fun( *this,
					&ExampleWindow::on_channel_change), radio_menu_items[i], i ) );
	}
   
    
    
    
    about_subitem = Gtk::manage(new Gtk::MenuItem(_("About Chord Sequencer") ) );
    application_submenu->append(*about_subitem);
    
    quit_subitem = Gtk::manage(new Gtk::MenuItem(_("Quit") ) );
    application_submenu->append(*quit_subitem);
    
    sequence_menu_item = Gtk::manage(new Gtk::MenuItem(_("Sequence") ) );
    sequence_submenu = Gtk::manage(new Gtk::Menu);
    sequence_menu_item->set_submenu(*sequence_submenu);
    
    select_seq_subitem = Gtk::manage(new Gtk::MenuItem(_("Select Sequence") ) );
    sequence_submenu->append(*select_seq_subitem);
    
    new_seq_subitem = Gtk::manage(new Gtk::MenuItem(_("New Sequence") ) );
    sequence_submenu->append(*new_seq_subitem);
    
    save_seq_subitem = Gtk::manage(new Gtk::MenuItem(_("Save Sequence") ) );
    sequence_submenu->append(*save_seq_subitem);
    
    rename_seq_subitem = Gtk::manage(new Gtk::MenuItem(_("Rename Sequence") ) );
    sequence_submenu->append(*rename_seq_subitem);
    
    chord_menu_item = Gtk::manage(new Gtk::MenuItem(_("Chord") ) );
    chord_submenu = Gtk::manage(new Gtk::Menu);
    chord_menu_item->set_submenu(*chord_submenu);
    
    new_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("New Chord" ) ) );
    chord_submenu->append(*new_chord_subitem);
    new_chord_subitem->set_hexpand( true);
      
    copy_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Copy Chord") ) );
    chord_submenu->append(*copy_chord_subitem);
    copy_chord_subitem->set_hexpand( true);
    
    paste_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Paste Chord") ) );
    chord_submenu->append(*paste_chord_subitem);
    paste_chord_subitem->set_hexpand( true);
    
    delete_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Delete Chord") ) );
    chord_submenu->append(*delete_chord_subitem);
    delete_chord_subitem->set_hexpand( true);
    
    select_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Select Chord") ) );
    chord_submenu->append(*select_chord_subitem);
    select_chord_subitem->set_hexpand( true);
    
    play_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Play Chord") ) );
    chord_submenu->append(*play_chord_subitem);
    play_chord_subitem->set_hexpand( true);
    
    next_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Next Chord") ) );
    chord_submenu->append(*next_chord_subitem);
    next_chord_subitem->set_hexpand( true);
    
    previous_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Previous Chord") ) );
    chord_submenu->append(*previous_chord_subitem);
    previous_chord_subitem->set_hexpand( true);
    
    stop_chord_subitem = Gtk::manage(new Gtk::MenuItem(_("Stop All Sounds") ) );
    chord_submenu->append(*stop_chord_subitem);
    stop_chord_subitem->set_hexpand( true);
    
    note_menu_item = Gtk::manage(new Gtk::MenuItem(_("Note") ) );
    note_submenu = Gtk::manage(new Gtk::Menu);
    note_menu_item->set_submenu(*note_submenu);
    
    new_note_subitem = Gtk::manage(new Gtk::MenuItem(_("New Note") ) );
    note_submenu->append(*new_note_subitem);
    new_note_subitem->set_hexpand( true);
    
    delete_note_subitem = Gtk::manage(new Gtk::MenuItem(_("Delete Note") ) );
    note_submenu->append(*delete_note_subitem);
    delete_note_subitem->set_hexpand( true);
    
    select_note_subitem = Gtk::manage(new Gtk::MenuItem(_("Select Note") ) );
    note_submenu->append(*select_note_subitem);
    select_note_subitem->set_hexpand( true);
    
    deselect_note_subitem = Gtk::manage(new Gtk::MenuItem(_("Deselect Note") ) );
    note_submenu->append(*deselect_note_subitem);
    deselect_note_subitem->set_hexpand( true);
    
    help_menu_item = Gtk::manage(new Gtk::MenuItem(_("Help") ) );
    help_submenu = Gtk::manage(new Gtk::Menu);
    help_menu_item->set_submenu(*help_submenu);
    
    manual_subitem = Gtk::manage(new Gtk::MenuItem(_("Read Manual") ) );
    help_submenu->append(*manual_subitem);
    manual_subitem->set_hexpand( true);
    
 	menubar = Gtk::manage(new Gtk::MenuBar);
    menubar->append(*application_menu_item);
    menubar->append(*sequence_menu_item);
    menubar->append(*chord_menu_item);
    menubar->append(*note_menu_item);
    menubar->append(*help_menu_item);
    menu_box.add(*menubar);
    
    
    
    main_VBox.pack_start(menu_box,  Gtk::PACK_SHRINK, 0);
    
    menu_items_with_shotrcuts[0] = play_chord_subitem;
    menu_items_with_shotrcuts[1] = previous_chord_subitem;
    menu_items_with_shotrcuts[2] = stop_chord_subitem;
    menu_items_with_shotrcuts[3] = new_chord_subitem;
    menu_items_with_shotrcuts[4] = copy_chord_subitem;
    menu_items_with_shotrcuts[5] = paste_chord_subitem;
 
	
	// connect actions and create shortcuts
	
	shortcuts_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_keyboard_shortcuts) ) );
	about_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::about_sequencer) ) );
	quit_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_button_quit) ) );
	select_seq_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_select_sequence) ) );		
	new_seq_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::new_sequence) ) );
	save_seq_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_save) ) );
	rename_seq_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_save) ) );
	
	new_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_new_chord) ) );
	copy_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_copy_chord) ) );
	paste_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_paste_chord) ) );
	delete_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_delete_chord) ) );
	select_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_select_chord) ) );
	play_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_play_chord) ) );
	next_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_next_chord) ) );
	previous_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_previous_chord) ) );
	stop_chord_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_stop_chord) ) );
	select_note_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_select_note) ) );				
	deselect_note_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_deselect_note) ) );
	new_note_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_new_note) ) );
	delete_note_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::on_action_delete_note) ) );
	manual_subitem->signal_activate().connect(
					(sigc::mem_fun(*this, &ExampleWindow::show_manual) ) );

	
	// set labels for subitems that have a shortcut
	update_menu_labels();
	// update labels when shortcuts changed
	ks_window.signal_hide().connect(sigc::mem_fun(*this,
							&ExampleWindow::update_menu_labels) );
	   
    			
		
	fixed_board.set_size_request(944, 626);
	
	main_VBox.pack_start(fixed_board,  Gtk::PACK_SHRINK, 0);
	//main_VBox.pack_start(note_chord_play_radio_box, Gtk::PACK_SHRINK, 0);
	play_note_rad.inner->set_size_request(100, 20);
	play_chord_rad.inner->set_size_request(100, 20);
	dont_play_rad.inner->set_size_request(100, 20);
	spacer_rad.inner->type = 1; // spacer
	play_note_rad.inner->type = 2; 
	play_note_rad.inner->name = 1;
	play_chord_rad.inner->type = 2;
	play_chord_rad.inner->name = 2;
	dont_play_rad.inner->type = 2;
	dont_play_rad.inner->name = 3;
	
	// name enumerated here
	/////////////////////////
	sharpsharp_rad.inner->name = 6;
	sharp_rad.inner->name = 7;
	normal_rad.inner->name = 8;
	flat_rad.inner->name = 9;
	flatflat_rad.inner->name = 10;
	
	display_group[4] = sharpsharp_rad;
	display_group[3] = sharp_rad;
	display_group[2] = normal_rad;
	display_group[1] = flat_rad;
	display_group[0] = flatflat_rad;
	
	for(int i=0; i<5; i++)
		display_group[i].inner->type = 2;
	for(int i=0; i<5; i++)
		display_group[i].inner->set_size_request(75, 20);
	normal_rad.inner->active = true;	
	
	note_label.inner->type = 4;
	note_label.inner->name = 4;
	instrument_label.inner->type = 4;
	instrument_label.inner->name = 5;
	instrument_label.inner->set_size_request(80, 25);
	note_label.inner->set_size_request(80, 25);
	
	spacer_rad.inner->set_size_request(100, 15);
	note_chord_play_radio_box.pack_start(*spacer_rad.e_box, Gtk::PACK_SHRINK, 0);
	note_chord_play_radio_box.pack_start(*play_note_rad.e_box, Gtk::PACK_SHRINK, 0);
	note_chord_play_radio_box.pack_start(*play_chord_rad.e_box, Gtk::PACK_SHRINK, 0);
	note_chord_play_radio_box.pack_start(*dont_play_rad.e_box, Gtk::PACK_SHRINK, 0);	
	dont_play_rad.inner->active = true;
	fixed_board.put(note_chord_play_radio_box, 540, 538);
	
	fixed_board.put(scroll, 96, 0);
	
	for (int i=0; i<64; i++)
	{
		chord_buttons[i] = new Gtk::EventBox();
		chord_radio_buttons[i] = new MyButton();
		chord_radio_buttons[i]->set_size_request(53, 12);
		chord_radio_buttons[i]->button_type = 1; // chord_radio_button
		button_chords_HBox.pack_start(*chord_buttons[i], Gtk::PACK_SHRINK, 0 );
		chord_buttons[i]->add(*chord_radio_buttons[i]);
	}
	chord_radio_buttons[0]->pressed = true;
	chord_radio_buttons[0]->occupied = true;
	
	
	// Modal buttons
	mode_note_edit_button->set_size_request(80, 24);
	mode_sequence_button->set_size_request(80, 24);
	mode_chord_edit_button->set_size_request(80, 24);
	mode_chord_play_button->set_size_request(80, 24);
	mult_select_b->set_size_request(80, 24);
	mult_select_b->set_margin_top(5);
	
	mode_note_edit_button->button_name = 3;
	mode_sequence_button->button_name = 4;
	mode_chord_edit_button->button_name = 1;
	mode_chord_play_button->button_name = 2;
	mult_select_b->button_name = 5;
	
	mode_note_edit_button->button_type = 2; // modal button
	mode_sequence_button->button_type = 2; // modal button
	mode_chord_edit_button->button_type = 2; // modal button
	mode_chord_play_button->button_type = 2; // modal button 
	mult_select_b->button_type = 3; 
	
	
	modal_button_VBox.pack_start(modal_button_HBox, Gtk::PACK_SHRINK, 0);
	modal_button_HBox.pack_start(*box_chord_edit, Gtk::PACK_SHRINK, 4);
	box_chord_edit->add(*mode_chord_edit_button);
	modal_button_HBox.pack_start(*box_chord_play, Gtk::PACK_SHRINK, 4);
	box_chord_play->add(*mode_chord_play_button);
	modal_button_HBox.pack_start(*box_note_edit, Gtk::PACK_SHRINK, 4); 
	box_note_edit->add(*mode_note_edit_button);
	modal_button_HBox.pack_start(*box_sequence, Gtk::PACK_SHRINK, 4); 
	box_sequence->add(*mode_sequence_button);
	
	modal_button_VBox.pack_start(scale_HBox,  Gtk::PACK_SHRINK, 0);
	scale_HBox.pack_start(*sc, Gtk::PACK_SHRINK, 6);
	box_mult_select->add(*mult_select_b);
	scale_HBox.pack_start(set_velocity_b, Gtk::PACK_SHRINK, 0);
	scale_HBox.pack_start(*box_mult_select, Gtk::PACK_SHRINK, 5);
	
	set_velocity_b.set_size_request(70, 24);
	set_velocity_b.set_vexpand(false);
	set_velocity_b.set_margin_top(5);	
	
	modal_button_VBox.set_size_request(315, 60);
	fixed_board.put(modal_button_VBox, 96, 557);
	// edit chord buttons
	
	chord_edit_button_box.pack_end(paste_chord_button, Gtk::PACK_SHRINK);
	chord_edit_button_box.pack_end(copy_chord_button, Gtk::PACK_SHRINK);
	chord_edit_button_box.pack_end(delete_chord_button, Gtk::PACK_SHRINK);
	chord_edit_button_box.pack_end(new_chord_button, Gtk::PACK_SHRINK);
	
	chord_edit_button_box.set_size_request(350, 60);
	fixed_board.put(chord_edit_button_box,  428, 557);
	
	// save and quit buttons
	m_Button_Quit.set_size_request( 96, 30);
	fixed_board.put(m_Button_Quit, 844, 591);
	
	save_button.set_size_request( 96, 30);
	fixed_board.put(save_button, 844, 557);
	
	// play chord buttons
	
	chord_play_button_box.pack_end(next_chord_button, Gtk::PACK_SHRINK);
	chord_play_button_box.pack_end(previous_chord_button, Gtk::PACK_SHRINK);
	chord_play_button_box.pack_end(play_button, Gtk::PACK_SHRINK);
	chord_play_button_box.pack_end(stop_button, Gtk::PACK_SHRINK);
	
	chord_play_button_box.set_size_request(350, 30);
	fixed_board.put(chord_play_button_box,  408, 557);

	instrument_HBox.pack_start(*instrument_label.e_box, Gtk::PACK_SHRINK, 4);
	
	// Combo Box code:
		//Create the Tree model:
	m_refTreeModel = Gtk::ListStore::create(m_Columns);
	inst_combo.set_model(m_refTreeModel);
	inst_combo.set_size_request(75, -1);
	Gtk::TreeModel::Row row;
	
	//Fill the ComboBox’s Tree Model: 53 instruments
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 1;
	row[m_Columns.m_col_name] = "Vibraphone";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 2;
	row[m_Columns.m_col_name] = "Soft Vibes";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 3;
	row[m_Columns.m_col_name] = "Simple Chimes";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 4;
	row[m_Columns.m_col_name] = "Silver Bell";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 6;
	row[m_Columns.m_col_name] = "Ethereal";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 12;
	row[m_Columns.m_col_name] = "Full Square";
	
	//row = *(m_refTreeModel->append());
	//row[m_Columns.m_col_id] = 14;
	//row[m_Columns.m_col_name] = "Steel Wire";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 15;
	row[m_Columns.m_col_name] = "Echo Bubbles";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 17;
	row[m_Columns.m_col_name] = "Sub Delay";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 18;
	row[m_Columns.m_col_name] = "Soft";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 19;
	row[m_Columns.m_col_name] = "Ultra Soft";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 23;
	row[m_Columns.m_col_name] = "Full Strings";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 24;
	row[m_Columns.m_col_name] = "Slow Strings";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 26;
	row[m_Columns.m_col_name] = "Sweep Synth";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 27;
	row[m_Columns.m_col_name] = "Warm Strings";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 30;
	row[m_Columns.m_col_name] = "Multi Synth";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 32;
	row[m_Columns.m_col_name] = "Medium Saw";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 35;
	row[m_Columns.m_col_name] = "Sweep Saw";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 39;
	row[m_Columns.m_col_name] = "MasterSynthLow";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 40;
	row[m_Columns.m_col_name] = "MasterSynthHi";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 41;
	row[m_Columns.m_col_name] = "Sharp Synth";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 45;
	row[m_Columns.m_col_name] = "Synth Bass";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 47;
	row[m_Columns.m_col_name] = "Variable Reed";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 48;
	row[m_Columns.m_col_name] = "Reed Organ";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 49;
	row[m_Columns.m_col_name] = "Soft Organ";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 51;
	row[m_Columns.m_col_name] = "Great Organ";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 52;
	row[m_Columns.m_col_name] = "64ft Organ";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 53;
	row[m_Columns.m_col_name] = "Hammond Organ";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 55;
	row[m_Columns.m_col_name] = "Sines";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 57;
	row[m_Columns.m_col_name] = "Warm Flute";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 59;
	row[m_Columns.m_col_name] = "Beyond";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 63;
	row[m_Columns.m_col_name] = "SlowDeepMatrix";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 64;
	row[m_Columns.m_col_name] = "Ice Field";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 65;
	row[m_Columns.m_col_name] = "HollowIceField";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 66;
	row[m_Columns.m_col_name] = "Smooth";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 67;
	row[m_Columns.m_col_name] = "Rushes";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 73;
	row[m_Columns.m_col_name] = "Sweep Rushes";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 76;
	row[m_Columns.m_col_name] = "Voyager";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 78;
	row[m_Columns.m_col_name] = "SoftVoyagerRev";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 81;
	row[m_Columns.m_col_name] = "Sharp and Warm";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 82;
	row[m_Columns.m_col_name] = "SharpAndDeep";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 87;
	row[m_Columns.m_col_name] = "Aooww";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 97;
	row[m_Columns.m_col_name] = "Wet Brass";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 98;
	row[m_Columns.m_col_name] = "Wet Brass 2";
		
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 114;
	row[m_Columns.m_col_name] = "Space Voice";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 115;
	row[m_Columns.m_col_name] = "FarSpaceVoice";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 116;
	row[m_Columns.m_col_name] = "SpaceVoice 2";
	
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 117;
	row[m_Columns.m_col_name] = "Soft Choir";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 118;
	row[m_Columns.m_col_name] = "Full Choir";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 119;
	row[m_Columns.m_col_name] = "Soft Ahh";

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 129;
	row[m_Columns.m_col_name] = "Breathy Ahh";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 121;
	row[m_Columns.m_col_name] = "Another Choir";
	
	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 122;
	row[m_Columns.m_col_name] = "Light Choir";	
	
		//Add the model columns to the Combo (which is a kind of view),
		//rendering them in the default way:
	inst_combo.pack_start(m_Columns.m_col_id);
	inst_combo.pack_start(m_Columns.m_col_name);
	instrument_HBox.pack_start(inst_combo, Gtk::PACK_SHRINK, 4);
	instrument_HBox.set_size_request(320, 30);
	fixed_board.put(instrument_HBox, 470, 592);
	
		//Connect signal handler:
	inst_combo.signal_changed().connect( sigc::mem_fun(*this, &ExampleWindow::on_inst_combo_changed) );	

	separator1.inner->type = 5;
	separator2.inner->type = 5;
	
	separator1.inner->set_size_request(4, 60);
	separator2.inner->set_size_request(4, 60);
	fixed_board.put(*separator1.e_box, 723, 557);
	fixed_board.put(*separator2.e_box, 834, 557);
	
	// m_ButtonHBox.pack_start(display_VBox,  Gtk::PACK_SHRINK, 4);
	for(int i=4; i>-1; i--)
		display_VBox.pack_start(*display_group[i].e_box, Gtk::PACK_SHRINK, 0);
	
	display_VBox.set_size_request(100, 60);
	fixed_board.put(display_VBox, 732, 557);	
		
	fixed_board.put(*boxes[77],  0, 0);
	boxes[77]->add(*keys[77]);
	keys[77]->set_size_request(62, 18);
	keys[77]->key_number = 77;
	
	fixed_board.put(*boxes[78],  62, 0);
	boxes[78]->add(*keys[78]);
	keys[78]->set_size_request(33, 18);
	keys[78]->key_number = 78;
	
	pairs[29].first = 77;
	pairs[29].second = 78;
	
	// Loop 4 times, once for each octave
	for (int i = 3, j = 0; i> -1; i--, j++)
	{
		int offset = 19 * i; 
		int pair_offset = 7 * i;
		int octave_offset = 126 * j;
		
		fixed_board.put(*boxes[19 + offset], 62, 18 + octave_offset);
		boxes[19 + offset]->add(*keys[19 + offset]);
		keys[19 + offset]->set_size_request(33, 18);
		keys[19 + offset]->key_number = 19 + offset;
		
		fixed_board.put(*boxes[18 + offset], 0, 18 + octave_offset);
		boxes[18 + offset]->add(*keys[18 + offset]);
		keys[18 + offset]->set_size_request(62, 9);
		keys[18 + offset]->key_number = 18 + offset;
		
		pairs[28 - pair_offset].first = 18 + offset;
		pairs[28 - pair_offset].second = 19 + offset;
		
		fixed_board.put(*boxes[17 + offset], 0, 27 + octave_offset);
		boxes[17 + offset]->add(*keys[17 + offset]);
		keys[17 + offset]->set_size_request(62, 12);
		keys[17 + offset]->key_number = 17 + offset;

		fixed_board.put(*boxes[16 + offset], 62, 36 + octave_offset);
		boxes[16 + offset]->add(*keys[16 + offset]);
		keys[16 + offset]->set_size_request(33, 18);
		keys[16 + offset]->key_number = 16 + offset;
		
		fixed_board.put(*boxes[15 + offset], 0, 39 + octave_offset);
		boxes[15 + offset]->add(*keys[15 + offset]);
		keys[15 + offset]->set_size_request(62, 9);
		keys[15 + offset]->key_number = 15 + offset;
		
		pairs[27 - pair_offset].first = 15 + offset;
		pairs[27 - pair_offset].second = 16 + offset;
		
		fixed_board.put(*boxes[14 + offset], 0, 48 + octave_offset);
		boxes[14 + offset]->add(*keys[14 + offset]);
		keys[14 + offset]->set_size_request(62, 12);
		keys[14 + offset]->key_number = 14 + offset;
		
		fixed_board.put(*boxes[13 + offset], 62, 54 + octave_offset);
		boxes[13 + offset]->add(*keys[13 + offset]);
		keys[13 + offset]->set_size_request(33, 18);
		keys[13 + offset]->key_number = 13 + offset;
		
		fixed_board.put(*boxes[12 + offset], 0, 60 + octave_offset);
		boxes[12 + offset]->add(*keys[12 + offset]);
		keys[12 + offset]->set_size_request(62, 9);
		keys[12 + offset]->key_number = 12 + offset;
		
		pairs[26 - pair_offset].first = 12 + offset;
		pairs[26 - pair_offset].second = 13 + offset;
		
		fixed_board.put(*boxes[11 + offset], 0, 69 + octave_offset);
		boxes[11 + offset]->add(*keys[11 + offset]);
		keys[11 + offset]->set_size_request(62, 12);
		keys[11 + offset]->key_number = 11 + offset;
		
		fixed_board.put(*boxes[10 + offset], 62, 72 + octave_offset);
		boxes[10 + offset]->add(*keys[10 + offset]);
		keys[10 + offset]->set_size_request(33, 18);
		keys[10 + offset]->key_number = 10 + offset;
		
		fixed_board.put(*boxes[9 + offset], 0, 81 + octave_offset);
		boxes[9 + offset]->add(*keys[9 + offset]);
		keys[9 + offset]->set_size_request(62, 9);
		keys[9 + offset]->key_number = 9 + offset;
		
		pairs[25 - pair_offset].first = 9 + offset;
		pairs[25 - pair_offset].second = 10 + offset;
		
		fixed_board.put(*boxes[8 + offset], 62, 90 + octave_offset);
		boxes[8 + offset]->add(*keys[8 + offset]);
		keys[8 + offset]->set_size_request(33, 18);
		keys[8 + offset]->key_number = 8 + offset;
		
		fixed_board.put(*boxes[7 + offset], 0, 90 + octave_offset);
		boxes[7 + offset]->add(*keys[7 + offset]);
		keys[7 + offset]->set_size_request(62, 11);
		keys[7 + offset]->key_number = 7 + offset;
		
		pairs[24 - pair_offset].first = 7 + offset;
		pairs[24 - pair_offset].second = 8 + offset;
		
		fixed_board.put(*boxes[6 + offset], 0, 101 + octave_offset);
		boxes[6 + offset]->add(*keys[6 + offset]);
		keys[6 + offset]->set_size_request(62, 12);
		keys[6 + offset]->key_number = 6 + offset;
		
		fixed_board.put(*boxes[5 + offset], 62, 108 + octave_offset);
		boxes[5 + offset]->add(*keys[5 + offset]);
		keys[5 + offset]->set_size_request(33, 18);
		keys[5 + offset]->key_number = 5 + offset;
		
		fixed_board.put(*boxes[4 + offset], 0, 113 + octave_offset);
		boxes[4 + offset]->add(*keys[4 + offset]);
		keys[4 + offset]->set_size_request(62, 9);
		keys[4 + offset]->key_number = 4 + offset;
		
		pairs[23 - pair_offset].first = 4 + offset;
		pairs[23 - pair_offset].second = 5 + offset;
		
		fixed_board.put(*boxes[3 + offset], 0, 122 + octave_offset);
		boxes[3 + offset]->add(*keys[3 + offset]);
		keys[3 + offset]->set_size_request(62, 12);
		keys[3 + offset]->key_number = 3 + offset;
		
		fixed_board.put(*boxes[2 + offset], 62, 126 + octave_offset);
		boxes[2 + offset]->add(*keys[2 + offset]);
		keys[2 + offset]->set_size_request(33, 18);
		keys[2 + offset]->key_number = 2 + offset;
		
		fixed_board.put(*boxes[1 + offset], 0, 134 + octave_offset);
		boxes[1 + offset]->add(*keys[1 + offset]);
		keys[1 + offset]->set_size_request(62, 10);
		keys[1 + offset]->key_number = 1 + offset;
		
		pairs[22 - pair_offset].first = 1 + offset;
		pairs[22 - pair_offset].second = 2 + offset;
	}
	// Start events section


	for(int i = 1; i<79; i++)
	{
		boxes[i]->set_events(Gdk::BUTTON_PRESS_MASK |
								Gdk::ENTER_NOTIFY_MASK | 
								Gdk::LEAVE_NOTIFY_MASK );
		boxes[i]->signal_button_press_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_key_pressed), i) );
		boxes[i]->signal_leave_notify_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_box_leave), i) );
		boxes[i]->signal_enter_notify_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_box_enter), i) );
    }
    
    for(int i=0; i<64; i++)
    {
		chord_buttons[i]->set_events(Gdk::BUTTON_PRESS_MASK |
								Gdk::ENTER_NOTIFY_MASK | 
								Gdk::LEAVE_NOTIFY_MASK );
		chord_buttons[i]->signal_button_press_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_chord_button_pressed), i) );
		chord_buttons[i]->signal_leave_notify_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_chord_button_leave), i) );
		chord_buttons[i]->signal_enter_notify_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_chord_button_enter), i) );
	}
	
	for(int i=0; i<5; i++)
	{
		display_group[i].e_box->set_events(Gdk::BUTTON_PRESS_MASK);
		display_group[i].e_box->signal_button_press_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_display_radio_clicked), i) );
	}
	
	play_note_rad.e_box->set_events(Gdk::BUTTON_PRESS_MASK);
	play_chord_rad.e_box->set_events(Gdk::BUTTON_PRESS_MASK);
	dont_play_rad.e_box->set_events(Gdk::BUTTON_PRESS_MASK);
	play_note_rad.e_box->signal_button_press_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_play_radio_clicked), 1) );
	play_chord_rad.e_box->signal_button_press_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_play_radio_clicked), 2) );
	dont_play_rad.e_box->signal_button_press_event().connect(sigc::bind<int>(
					sigc::mem_fun(*this, &ExampleWindow::on_play_radio_clicked), 3) );
	
	area.signal_button_press_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_area_clicked) );
    	// standard buttons events here
    play_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_play_chord) );
    stop_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_stop_chord) );
    next_chord_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_next_chord) );
    previous_chord_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_previous_chord) );
    save_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_save) );
	copy_chord_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_copy_chord) );
    paste_chord_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_paste_chord) );
    delete_chord_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_delete_chord) );
    new_chord_button.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_new_chord) );
    set_velocity_b.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_set_velocity) );
	m_Button_Quit.signal_clicked().connect( sigc::mem_fun(*this, &ExampleWindow::on_button_quit) );
    /*////////////////////////////////////////////////////////////////////////////////////*/
    
    box_note_edit->set_events(Gdk::BUTTON_PRESS_MASK |	Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK );
	box_note_edit->signal_button_press_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_note_edit_clicked) );
	box_note_edit->signal_leave_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_note_edit_leave) );
	box_note_edit->signal_enter_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_note_edit_enter) );         
	
	box_sequence->set_events(Gdk::BUTTON_PRESS_MASK |	Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK );
	box_sequence->signal_button_press_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_sequence_clicked) );
	box_sequence->signal_leave_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_sequence_leave) );
	box_sequence->signal_enter_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_sequence_enter) );
	
	box_chord_edit->set_events(Gdk::BUTTON_PRESS_MASK |	Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK );
	box_chord_edit->signal_button_press_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_chord_edit_clicked) );
	box_chord_edit->signal_leave_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_chord_edit_leave) );
	box_chord_edit->signal_enter_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_chord_edit_enter) );
	
	box_chord_play->set_events(Gdk::BUTTON_PRESS_MASK |	Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK );
	box_chord_play->signal_button_press_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_chord_play_clicked) );
	box_chord_play->signal_leave_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_chord_play_leave) );
	box_chord_play->signal_enter_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_chord_play_enter) );
	
	box_mult_select->set_events(Gdk::BUTTON_PRESS_MASK |	Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK );
	box_mult_select->signal_button_press_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_mult_select_clicked) );
	box_mult_select->signal_leave_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_mult_select_leave) );
	box_mult_select->signal_enter_notify_event().connect(sigc::mem_fun(*this, &ExampleWindow::on_mult_select_enter) );
	
	// sequence window button events
	s_win.m_Button_Select.signal_clicked().connect(sigc::bind<Gtk::TreeView*>(
							sigc::mem_fun(*this, &ExampleWindow::select_sequence), &s_win.m_TreeView ) );
	s_win.m_Button_Delete.signal_clicked().connect(sigc::bind<Gtk::TreeView*>(
							sigc::mem_fun(*this, &ExampleWindow::delete_sequence), &s_win.m_TreeView ) );
	s_win.m_Button_New.signal_clicked().connect(
							sigc::mem_fun(*this, &ExampleWindow::new_sequence) );
	s_win.m_Button_Cancel.signal_clicked().connect(
							sigc::mem_fun(*this, &ExampleWindow::cancel_sequence) );						
	show_all_children();
	
	
	
	if(mode == 1)
	{
		chord_play_button_box.hide();
		note_chord_play_radio_box.hide();
		instrument_HBox.hide();
		scale_HBox.hide();
		display_VBox.hide();
		separator1.e_box->hide();
		//set_velocity_b.hide();
	}
	
	read_saved_file();
}

ExampleWindow::~ExampleWindow()
{
	
}

// Called when the user clicks "Sequence" button and then clicks "Cancel"
void ExampleWindow::cancel_sequence()
{
	s_win.hide();
}

// This function should be embedded in a try/catch block in case of
// an exception.  It offers the user a choice of MIDI ports to open.
// It returns false if there are no ports available.
bool ExampleWindow::chooseMidiPort( RtMidiOut *rtmidi )
{
	/*
	std::cout << "\nWould you like to open a virtual output port? [y/N] ";

	std::string keyHit;
	std::getline( std::cin, keyHit );
	if ( keyHit == "y" )
	{
		rtmidi->openVirtualPort();
		return true;
	}
	*/
	
	
	std::string portName;
	unsigned int i = 0, nPorts = rtmidi->getPortCount();
	if ( nPorts == 0 )
	{
		std::cout << _("No output ports available!") << std::endl;
		return false;
	}

	if ( nPorts == 1 )
	{
		std::cout << "\n" << _("Opening") << rtmidi->getPortName() << std::endl;
	}
	else
	{
		for ( i=0; i<nPorts; i++ )
		{
			portName = rtmidi->getPortName(i);
			std::cout << _("  Output port #")  << i << ": " << portName << '\n';
		}

		do
		{
			std::cout << "\n" << _("Choose a port number: ");
			std::cin >> i;
		}	
		while ( i >= nPorts );
	}

	std::cout << "\n";
	rtmidi->openPort( i );

	return true;
}


bool ExampleWindow::on_display_radio_clicked(GdkEventButton* button_event, int data)
{
	// change active button
	for(int i = 0; i<5; i++)
	{
		if(i == data)
			display_group[i].inner->active = true;
		else
			display_group[i].inner->active = false;
		display_group[i].inner->queue_draw();
	}
	
	// change selected note display
	selected_note->display = data;

	selected_note->update_display();
	selected_note->update_x();
	area.queue_draw();
	
	return true;
}


 /*)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))*/
 
// This function is called when modal button "edit note" clicked
bool ExampleWindow::on_note_edit_clicked(GdkEventButton* button_event)
{
	mode = 3;
	sharpsharp_rad.inner->hide();
	flatflat_rad.inner->hide();
	normal_rad.inner->hide();
	flat_rad.inner->hide();
	sharp_rad.inner->hide();
	
		
	chord_edit_button_box.hide();
	chord_play_button_box.hide();
	instrument_HBox.hide();
	display_VBox.show();
	note_chord_play_radio_box.show();
	separator1.e_box->show();
	scale_HBox.show();
	//sc->set_sensitive(true);
	//set_velocity_b.show();
	
	if(multiple_select == false && selected_note != NULL)
	{
		display_note_options(selected_note);
		sc->set_value(selected_note->velocity);
	}
	
	mode_chord_edit_button->pressed = false;
	mode_chord_play_button->pressed = false;
	mode_sequence_button->pressed = false; 
	mode_note_edit_button->pressed = true;
	
	mode_chord_edit_button->queue_draw ();
	mode_chord_play_button->queue_draw ();
	mode_sequence_button->queue_draw ();
	mode_note_edit_button->queue_draw ();
	return true;
}

// This function is called when mouse leaves modal button "edit note"
bool ExampleWindow::on_note_edit_leave(GdkEventCrossing* crossing_event)
{
	mode_note_edit_button->over = false;
	mode_note_edit_button->queue_draw ();
	return true;
}

// This function is called when mouse enters modal button "edit note"
bool ExampleWindow::on_note_edit_enter(GdkEventCrossing* crossing_event)
{
	mode_note_edit_button->over = true;
	mode_note_edit_button->queue_draw ();
	return true;
}
 /*)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))*/
 
bool ExampleWindow::on_sequence_clicked(GdkEventButton* button_event)
{
	(void*) button_event;
	mode_chord_edit_button->pressed = false;
	mode_chord_play_button->pressed = false;
	mode_sequence_button->pressed = true; 
	mode_note_edit_button->pressed = false;
	
	mode_chord_edit_button->queue_draw ();
	mode_chord_play_button->queue_draw ();
	mode_sequence_button->queue_draw ();
	mode_note_edit_button->queue_draw ();
	
	//set_velocity_b.hide();
	scale_HBox.hide();
	
	if(s_win.get_visible() )
		s_win.hide();
	else
	{
		mode = 4;
		s_win.send_data(sequence_names);
		s_win.show();
	}
	return true;
}

bool ExampleWindow::on_sequence_leave(GdkEventCrossing* crossing_event)
{
	mode_sequence_button->over = false;
	mode_sequence_button->queue_draw ();
	return true;
}

bool ExampleWindow::on_sequence_enter(GdkEventCrossing* crossing_event)
{
	mode_sequence_button->over = true;
	mode_sequence_button->queue_draw ();
	return true;
}	
 /*)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))*/
 
// this function is called when modal "Edit chord" button clicked
bool ExampleWindow::on_chord_edit_clicked(GdkEventButton* button_event)
{
	//puts("clicked");
	mode = 1; // chord edit mode
	
	instrument_HBox.hide();
	display_VBox.hide();
	separator1.e_box->hide();
	chord_play_button_box.hide();
	note_chord_play_radio_box.hide();
	//sc->set_sensitive(false);
	//set_velocity_b.hide();
	scale_HBox.hide();
	chord_edit_button_box.show();
	chord_edit_button_box.show_all_children();
	
	mode_chord_edit_button->pressed = true;
	mode_chord_play_button->pressed = false;
	mode_sequence_button->pressed = false; 
	mode_note_edit_button->pressed = false;
	
	mode_chord_edit_button->queue_draw ();
	mode_chord_play_button->queue_draw ();
	mode_sequence_button->queue_draw ();
	mode_note_edit_button->queue_draw ();
	return true;
}

// this function is called when mouse leaves modal "Edit chord" button
bool ExampleWindow::on_chord_edit_leave(GdkEventCrossing* crossing_event)
{
	mode_chord_edit_button->over = false;
	mode_chord_edit_button->queue_draw ();
	return true;
}

// this function is called when mouse enters modal "Edit chord" button
bool ExampleWindow::on_chord_edit_enter(GdkEventCrossing* crossing_event)
{
	mode_chord_edit_button->over = true;
	mode_chord_edit_button->queue_draw ();
	return true;
}	
 /*)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))*/
 
// This function is called when modal button "play chord" clicked  
bool ExampleWindow::on_chord_play_clicked(GdkEventButton* button_event)
{
	(void*) button_event;
	//puts("clicked");
	mode = 2; // chord play mode
	separator1.e_box->hide();
	chord_edit_button_box.hide();
	note_chord_play_radio_box.hide();
	display_VBox.hide();
	instrument_HBox.show();
	chord_play_button_box.show();
	chord_play_button_box.show_all_children();
	//sc->set_sensitive(false);
	//set_velocity_b.hide();
	scale_HBox.hide();
	
	mode_chord_edit_button->pressed = false;
	mode_chord_play_button->pressed = true;
	mode_sequence_button->pressed = false; 
	mode_note_edit_button->pressed = false;
	
	mode_chord_edit_button->queue_draw ();
	mode_chord_play_button->queue_draw ();
	mode_sequence_button->queue_draw ();
	mode_note_edit_button->queue_draw ();
	return true;
}

// This function is called when mouse leaves modal button "play chord"
bool ExampleWindow::on_chord_play_leave(GdkEventCrossing* crossing_event)
{
	mode_chord_play_button->over = false;
	mode_chord_play_button->queue_draw ();
	return true;
}

// This function is called when mouse moves over modal button "play chord"
bool ExampleWindow::on_chord_play_enter(GdkEventCrossing* crossing_event)
{
	mode_chord_play_button->over = true;
	mode_chord_play_button->queue_draw ();
	return true;
}	
/*)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))*/
// This function is called when one of 64 buttons below the area clicked
bool ExampleWindow::on_chord_button_pressed(GdkEventButton* button_event, int data)
{
	//printf("Pressed button %d\n", data);
	int length = g_list_length(area.current_sequence);
		
	if(data < length )
	{
	// change current chord
	selected_chord_number = data + 1; // start counting chords from 1
	// still counting from 0 here:
	area.current_chord = (GList * ) g_list_nth_data (area.current_sequence, data);
	
	//printf("Selected chord %d\n", data + 1);
	
		// mark button as selected
		chord_radio_buttons[data]->pressed = true;
		for (int i=0; i<64; i++)
		{
			if (i != data)
			{
				chord_radio_buttons[i]->pressed = false;
			}
			chord_radio_buttons[i]->queue_draw (); 
		}
	}
	area.queue_draw();
	return true;
}

// This function is called when mouse leaves one of 64 buttons below the area
bool ExampleWindow::on_chord_button_leave(GdkEventCrossing* crossing_event, int data)
{
	chord_radio_buttons[data]->over = false;
	chord_radio_buttons[data]->queue_draw ();
	return true;
}

// This function is called when mouse moves over one of 64 buttons below the area
bool ExampleWindow::on_chord_button_enter(GdkEventCrossing* crossing_event, int data)
{
	chord_radio_buttons[data]->over = true;
	chord_radio_buttons[data]->queue_draw ();
	return true;
}	
/*)))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))) */

// This function is called when "new chord" button is pressed
void ExampleWindow::on_new_chord()
{
	int length = g_list_length (area.current_sequence);
	
	// every chord has at least one note, dummy note, which 
	// is not counted or displayed
	GList * new_chord = new GList();
	Note * dummy_note = new Note(-1000);
	new_chord->data = dummy_note;
	
	// find out which of the chords is selected:
	// and insert new chord AFTER it.
	// find out if selected chord is the last one:
	if(length == selected_chord_number) // selected chord is last one
		// this returns the new start of the list
		// and adds new chord to the end of the list
		area.current_sequence = g_list_insert_before(area.current_sequence, NULL, new_chord);
	else
	{	
		// this points to the next chord after selected
		GList * after_selected_chord = g_list_nth (area.current_sequence, selected_chord_number);
		// this returns the new start of the list
		area.current_sequence = g_list_insert_before(area.current_sequence, after_selected_chord, new_chord);
	}
	
	// make new chord selected
	selected_chord_number++;
	// 0 start notation
	area.current_chord = (GList * ) g_list_nth_data (area.current_sequence, selected_chord_number - 1);
	// mark corresponding chord buttons - 0 start notation
	chord_radio_buttons[selected_chord_number - 2]->pressed = false;
	chord_radio_buttons[selected_chord_number - 1]->pressed = true;
	chord_radio_buttons[selected_chord_number - 2]->queue_draw();
	chord_radio_buttons[selected_chord_number - 1]->queue_draw();
	
	// change position_x for all notes in all chords to the right of new one
	// change position_x for all notes in all chords to the right of new one
	GList * buffer;
	Note * n;
	for(int i=selected_chord_number; i< g_list_length(area.current_sequence); i++)
	{
		buffer = (GList *) g_list_nth (area.current_sequence, i)->data;
		for(int j=1; j< g_list_length(buffer); j++)
		{
			n = (Note *) (g_list_nth(buffer, j)->data);
			(n->position_x) += 53;
		}
	}
	
	// update all occupied fields in chord buttons
	int len = g_list_length(area.current_sequence);
	for(int i=0; i<64; i++)
	{
		if(i < len)
			chord_radio_buttons[i]->occupied = true;
		else
			chord_radio_buttons[i]->occupied = false;
		chord_radio_buttons[i]->queue_draw();
	}
	area.queue_draw();
}

void ExampleWindow::on_delete_chord()
{
	int length = g_list_length(area.current_sequence);
	
	GList * previous_sequence_start = area.current_sequence;
	if(g_list_length(area.current_sequence) == 1)
	{	 // sequence always must have at least one chord
		 // delete notes only, leaving buffer note
		GList * buffer = (GList *) g_list_nth (area.current_sequence, selected_chord_number - 1 )->data;
		int length = g_list_length(buffer);
		for(int i=1; i < length; i++)
		{
	//		printf("Deleting note # %d\n", i);
			delete ( (Note *) g_list_nth (buffer, 1)->data); 
			buffer = g_list_delete_link(buffer, g_list_nth(buffer, 1) );
		}
		area.queue_draw();
		return;
	}
		
	// If we reached this line, that means there is more than 1 chord in sequence

	// delete all notes first
	GList * buffer = (GList *) g_list_nth (area.current_sequence, selected_chord_number - 1 )->data;
	for(int i=0; i < g_list_length(buffer); i++)
		delete ( (Note *) g_list_nth (buffer, i)->data);
	// delete GList
	area.current_sequence = g_list_delete_link(area.current_sequence, 
			g_list_nth (area.current_sequence, selected_chord_number - 1 ) );
	// change selected_chord_number and the chord itself
	if(g_list_length(area.current_sequence) == selected_chord_number - 1)
	{
		// means no more chords to the right
		selected_chord_number--;
		// 0 start notation
		area.current_chord = (GList * ) g_list_nth_data (area.current_sequence, selected_chord_number - 1);
		// mark corresponding chord buttons - 0 start notation
		chord_radio_buttons[selected_chord_number]->pressed = false;
		chord_radio_buttons[selected_chord_number - 1]->pressed = true;
		chord_radio_buttons[selected_chord_number]->queue_draw();
		chord_radio_buttons[selected_chord_number - 1]->queue_draw();
	}
	else // means there are chords to the right
	{
		// selected_chord_number is unchanged, changing chord itself 
		// to one to the right of the deleted
		area.current_chord = (GList * ) g_list_nth_data (area.current_sequence, selected_chord_number - 1);
	
	 
		// change position_x in all notes to the right of deleted chord
		GList * buffer;
		Note * n;
		for(int i=selected_chord_number - 1; i< g_list_length(area.current_sequence); i++)
		{
			buffer = (GList *) g_list_nth (area.current_sequence, i)->data;
			for(int j=1; j< g_list_length(buffer); j++)
			{
				n = (Note *) (g_list_nth(buffer, j)->data);
				(n->position_x) -= 53;
			}
		}
 	}
 		
	if(selected_chord_number == 1)
	{
		// reconnect new sequence start to the area.sequences GList
		int i = g_list_index (area.sequences, previous_sequence_start);
	//	printf("index i is %d\n", i);
		g_list_nth(area.sequences, i)->data = area.current_sequence;
	}
 	
 		// update all occupied fields in chord buttons
	int len = g_list_length(area.current_sequence);
	for(int i=0; i<64; i++)
	{
		if(i < len)
			chord_radio_buttons[i]->occupied = true;
		else
			chord_radio_buttons[i]->occupied = false;
		chord_radio_buttons[i]->queue_draw();
	}
 	
	area.queue_draw();
}

void ExampleWindow::on_button_quit()
{
	for (int i=0; i<79; i++)
	{
		delete  boxes[i];
	}
	for (int i=0; i<83; i++)
	{
		delete keys[i];
	}
	delete midiout;
	delete selected_notes;
	delete dummy_selected_note;
	delete mode_note_edit_button;
	delete mode_sequence_button;
	delete mode_chord_edit_button;
	delete mode_chord_play_button;
	delete mult_select_b;
	delete box_note_edit;
	delete box_sequence;
	delete box_chord_edit;
	delete box_chord_play;
	delete box_mult_select;
	delete sc;
	for(int i=0; i<64; i++)
	{
		delete chord_buttons[i];
		delete chord_radio_buttons[i];
	}
	
	exit(0);
}

// finds midi number based on pressed key
int ExampleWindow::findMidiNumber(int number)
{
	int octave; // 0 for the bottom octave
	octave = (number - 1 - (number - 1) % 19) / 19 ;
	
	switch (number - octave * 19)
	{
		case 1: case 2:	
			return 36 + octave * 12;
		case 3: 		
			return 37 + octave * 12;
		case 4: case 5: 
			return 38 + octave * 12;
		case 6:
			return 39 + octave * 12;
		case 7: case 8:
			return 40 + octave * 12;
		case 9: case 10:
			return 41 + octave * 12;
		case 11:
			return 42 + octave * 12;
		case 12: case 13:
			return 43 + octave * 12;
		case 14:
			return 44 + octave * 12;
		case 15: case 16:
			return 45 + octave * 12;
		case 17:
			return 46 + octave * 12;
		case 18: case 19:
			return 47 + octave * 12;
	}
}

// this function is called when one of the piano keys is pressed
bool ExampleWindow::on_key_pressed(GdkEventButton* button_event, int data)
{
	GList * buffer_list;
	// Find out the MIDI note number based on data:
	// The lowest C in this application is    36
	
	int number = findMidiNumber(data);
	bool note_exist = false;
	//printf("%d is pressed\n", number);
	
	// if note already exist in the currently selected chord, delete it
	Note * n;
	for(uint k=1; k<g_list_length (area.current_chord); k++)
	{
		n = (Note *) (g_list_nth(area.current_chord, k)->data);
		int midi_num = n->midi_number;
		//printf("Note number %d, position_y = %d\n", k, pos_y);
		if (number == midi_num)
		{	
			//puts("note found");
			// if note is selected, remove it from selected_notes GList
			if(n->selected)
			{
				int index = g_list_index (selected_notes, n);
				buffer_list = g_list_nth(selected_notes, index);
				selected_notes = g_list_delete_link(selected_notes, buffer_list);
			}
			note_exist = true;
			if(multiple_select == false && n == selected_note)
				selected_note = NULL;
			
			delete n;
			area.current_chord = g_list_delete_link(area.current_chord, 
						g_list_nth(area.current_chord, k) );
			break;
		}
	}
	
	if(! note_exist)
	{
		Note * note = new Note(number);
		note->normal_x = (selected_chord_number - 1) * 53 ;
		note->update_x();
		area.current_chord = g_list_append(area.current_chord, note ) ;
	}
	area.queue_draw();
	return true;
}

// this function is called when mouse moves over one of the piano keys
bool ExampleWindow::on_box_enter(GdkEventCrossing* crossing_event, int data )
{	
	int d = data;
	int temp1, temp2;
		
	for(int i=0; i<30; i++)
	{
		temp1 = pairs[i].first;
		temp2 = pairs[i].second;
		if ( temp1 == d  )
		{
			if(keys[temp2]->over == false)
			{
				keys[temp2]->over = true;
				keys[temp2]->queue_draw ();
			}
		}
		if ( temp2 == d )
		{
			if(keys[temp1]->over == false)
			{
				keys[temp1]->over = true;
				keys[temp1]->queue_draw ();
			}
		}
	}
	
	keys[d]->over = true;
	keys[d]->queue_draw ();
	return true;
}

// this function is called when mouse moves out of one of the piano keys
bool ExampleWindow::on_box_leave(GdkEventCrossing* crossing_event, int data)
{
	int d = data;
	int temp1, temp2;
	for(int i=0; i<30; i++)
	{
		temp1 = pairs[i].first;
		temp2 = pairs[i].second;
		if ( temp1 == d  )
		{
			if(keys[temp2]->over == true)
			{
				keys[temp2]->over = false;
				keys[temp2]->queue_draw ();
			}
		}
		if ( temp2 == d )
		{
			if(keys[temp1]->over == true)
			{
				keys[temp1]->over = false;
				keys[temp1]->queue_draw ();
			}
		}
	}

	keys[d]->over = false;
	keys[d]->queue_draw ();
	return true;
}

bool ExampleWindow::on_play_radio_clicked(GdkEventButton* button_event, int data)
{
	GList * buffer;
	Note * n;
	int note_number;
	int note_velocity;
	if(data == 1) // Play Note option
	{
		if(selected_note != NULL)
		{
			note_number = selected_note->midi_number;
			note_velocity = selected_note->velocity;
	
			// Note On: 144, 64, 90 - second channel
			message[0] = 144  + channel - 1;
			message[1] = note_number;
			message[2] = note_velocity;
			midiout->sendMessage( &message );
		}
		play_note_rad.inner->active = true;
		play_note_rad.inner->queue_draw();
		play_chord_rad.inner->active = false;
		play_chord_rad.inner->queue_draw();
		dont_play_rad.inner->active = false;
		dont_play_rad.inner->queue_draw();
	}
	else if(data == 2) // Play Chord
	{
		for( int i = 1; i < g_list_length(area.current_chord); i++)
		{
			n = (Note *) g_list_nth_data(area.current_chord, i);
			note_number = n->midi_number;
			note_velocity = n->velocity;
			message[0] = 144  + channel - 1;
			message[1] = note_number;
			message[2] = note_velocity;
			midiout->sendMessage( &message );
		}
		
		
		play_note_rad.inner->active = false;
		play_note_rad.inner->queue_draw();
		play_chord_rad.inner->active = true;
		play_chord_rad.inner->queue_draw();
		dont_play_rad.inner->active = false;
		dont_play_rad.inner->queue_draw();
	}
	else if(data == 3) // Don't Play option
	{
		// All notes off
		message[0] = 176  + channel - 1;
		message[1] = 123;
		message[2] = 0;
		midiout->sendMessage( &message );
		
		play_note_rad.inner->active = false;
		play_note_rad.inner->queue_draw();
		play_chord_rad.inner->active = false;
		play_chord_rad.inner->queue_draw();
		dont_play_rad.inner->active = true;
		dont_play_rad.inner->queue_draw();
	}

	return true;
}

// This function displays note options, which are possible choices of 
// how the note will be displayed (flat, sharp, double flat, double sharp
// or clear).
void ExampleWindow::display_note_options(Note * n)
{

	// Show corresponding display options
	int  options = n->display_options;
	int disp = n->display;
	int options_length;
	int opt;
	if(options >= 100) // 3 digits
	{
		options_length = 3;
		int opt1 = (options - options % 100) / 100;
		int tens = options - opt1 * 100;
		int opt2 = (tens - tens % 10) / 10;
		int opt3 = tens % 10;
		
		for(int i=4; i>-1; i--)
		{
			if( (i == opt1) || (i == opt2) || (i == opt3) )
				display_group[i].inner->show();
			else 
				display_group[i].inner->hide();
				
			if(disp == i)
				display_group[i].inner->active = true;
			else 
				display_group[i].inner->active = false;
			display_group[i].inner->queue_draw();
		}
		
	}
	else // 2 digits
	{
		options_length = 2;
		int opt1 = (options - options % 10) / 10;
		int opt2 = options % 10;
		
		for(int i=4; i>-1; i--)
		{
			if( (i == opt1) || (i == opt2) )
				display_group[i].inner->show();
			else 
				display_group[i].inner->hide();
				
			if(disp == i)
				display_group[i].inner->active = true;
			else 
				display_group[i].inner->active = false;
			display_group[i].inner->queue_draw();
		}
	}	
	//printf("Selected options inside on_area_clicked are %d\n", options);
}

// This function is called when the user clicks on existing note, which
// will either select or deselect it. It is also called when user misses 
// the note and clicks on empty area space.
bool ExampleWindow::on_area_clicked(GdkEventButton* button_event)
{
	//printf("Number of selected notes at the beginning: %d\n", g_list_length(selected_notes) );
	//printf("Address of selected_notes inside is %p\n", selected_notes);
	Note * candidate_note, * clicked_note;
	GList * buffer_list;
	bool found = false;
	int x = button_event->x;
	int y = button_event->y;
	int y_correction;
//	printf("Area clicked at %d, %d\n", x, y);
	int chord_index = (x - (x % 53) ) / 53; // starts from 0
//	printf("chord index is  %d\n", chord_index);
	
	// find if any chords are there at this x:
	int length = g_list_length (area.current_sequence);
	if( chord_index < length )
	{
		GList * clicked_chord = (GList *) (g_list_nth
					(area.current_sequence, chord_index)->data);
		//printf("This chord has %d notes\n", g_list_length(clicked_chord) );

			// find if any notes are there in this chord at this y:
		for(int i = 0; i < g_list_length (clicked_chord); i++) 
		{   
			
			candidate_note = (Note *) (g_list_nth
					(clicked_chord, i)->data);
			int candidate_y = candidate_note->position_y;
			int interval = 0;
			if( (candidate_y > 0 && candidate_y < 62) || (candidate_y > 420 ) )
			{ 		interval = 15;	y_correction = 12;}	
			else {  interval = 18;	y_correction = 12;}	
			
			candidate_y += y_correction;	
			
			//printf("Note %d, candidate_y is %d\n", i, candidate_y);
			if(y > candidate_y && y < candidate_y + interval)
			{
	//			puts("found note to be selected or deselected");
				clicked_note = candidate_note;
				found = true;
			}
	
		}
	} 
	
	if (found == false) // clicked on empty area
		return true;
	
	// Now that we found the selected note, let's do something with it
	if(clicked_note->selected == false)
	{
		if(multiple_select == false)
		{
			
			if(g_list_length(selected_notes) > 1) // there are already
											// selected notes in it
			{	
			// deselect previously selected note and make clicked note selected
				buffer_list = g_list_nth(selected_notes, 1);
				selected_note->selected = false;
				selected_notes = g_list_delete_link(selected_notes, buffer_list);
			
				clicked_note->selected = true;
				selected_note = clicked_note;	
				selected_notes = g_list_append(selected_notes, clicked_note ) ;
			}
			else // no selected notes yet
			{
				// add clicked note to selected_notes and make it selected
				clicked_note->selected = true;
				selected_notes = g_list_append(selected_notes, clicked_note );
				selected_note = clicked_note;
				if(mode == 3)
				{
					set_velocity_b.set_sensitive(true);
					sc->set_sensitive(true);
				}
			}
		
	//		printf("Number of selected notes: %d\n", g_list_length(selected_notes) );
			
			if(mode == 3) // Edit Note Mode
			{	// Due to the unique nature of select options for each
				// individual note, the only way to change of view them is
				// to use single note select mode.
	
				display_note_options(clicked_note);
				display_VBox.show();
					// set the velocity scale value
				sc->set_value(selected_note->velocity );
			}
		}
		else  // multiple_select is true
		{
			// add clicked note to selected_notes and make it selected
	//		puts("inside if multiple_select is true and note selected false");
			clicked_note->selected = true;
			selected_notes = g_list_append(selected_notes, clicked_note );
		}
	}
	else if(clicked_note->selected == true)
	{
		
		
		if( g_list_length(selected_notes) == 2 && multiple_select == false)
		// this means that if there is one selected note (without dummy note)
		// in the list of selected notes and it needs to be deselected, so 
		// there will be no selected notes on the screen. First argument is not
		// necessary, it is there just for clarification.
		{
			// only one note is selected in single select mode and it needs
			// to be deselected
			selected_note = NULL;
			set_velocity_b.set_sensitive(false);
			sc->set_sensitive(false);
			display_VBox.hide();
		}
		clicked_note->selected = false;
		int index = g_list_index (selected_notes, clicked_note);
		buffer_list = g_list_nth(selected_notes, index);
		selected_notes = g_list_delete_link(selected_notes, buffer_list);
		
			
	}
	area.queue_draw();
	return true;
}

bool ExampleWindow::on_mult_select_clicked(GdkEventButton* button_event)
{
	GList * buffer_list;
	Note * buffer_note;
	if(mult_select_b->pressed == false)
	{
		multiple_select = true;
		mult_select_b->pressed = true;
		
		// hide all display radio buttons
		sharpsharp_rad.inner->hide();
		flatflat_rad.inner->hide();
		normal_rad.inner->hide();
		flat_rad.inner->hide();
		sharp_rad.inner->hide();
		selected_note = NULL;
	}
	else if(mult_select_b->pressed == true)
	{
		multiple_select = false;
		mult_select_b->pressed = false;
		// deselect all notes
		while( g_list_length(selected_notes) > 1 )
		{	
			buffer_list = g_list_nth(selected_notes, 1);
			buffer_note = (Note *) (buffer_list->data);
			buffer_note->selected = false;
					
			selected_notes = g_list_delete_link(selected_notes, buffer_list);
		}
		selected_note = NULL;
	}
	mult_select_b->queue_draw ();
	area.queue_draw();
	return true;
}

bool ExampleWindow::on_mult_select_leave(GdkEventCrossing* crossing_event)
{
	mult_select_b->over = false;
	mult_select_b->queue_draw ();
	return true;
}

bool ExampleWindow::on_mult_select_enter(GdkEventCrossing* crossing_event)
{
	mult_select_b->over = true;
	mult_select_b->queue_draw ();
	return true;
}

void ExampleWindow::on_set_velocity()
{
	Note * n;
	
	if(multiple_select == false)
	{
		if(selected_note != NULL)
		{
			selected_note->velocity = adj->get_value();
			std::cout << (_("velocity set") ) << "\n";
		}
	}
	else // multiple select is true
	{
		for(int i=1; i<g_list_length(selected_notes); i++)
		{
			n = (Note *) g_list_nth_data(selected_notes, i);
			n->velocity = adj->get_value();
			std::cout << (_("velocity set") ) << "\n";
		}
	}
	
}

void ExampleWindow::on_inst_combo_changed()
{
	int id;
	Glib::ustring name;
	Gtk::TreeModel::iterator iter = inst_combo.get_active();
	
	if(iter)
	{
		Gtk::TreeModel::Row row = *iter;
		if(row)
		{
			//Get the data for the selected row, using our knowledge 
			//of the tree model:
			id = row[m_Columns.m_col_id];
			name = row[m_Columns.m_col_name];
						
			message[0] = 192  + channel - 1; 
			message[1] =  id;
			message.pop_back();
			midiout->sendMessage( &message );
			message.push_back(0);
	//		puts("inside combo changed");
		}
	}
	else
		std::cout << _("invalid combo iterator") << std::endl;
}

void ExampleWindow::on_copy_chord()
{
	// You cannot delete the original chord, if you do, paste will not work
	copied_chord = area.current_chord;	
}

void ExampleWindow::on_paste_chord()
{
	if(copied_chord != NULL)
	{	
		Note * buffer_note;
		int length = g_list_length(copied_chord);
		GList * buffer = area.current_chord;
		int buffer_length = g_list_length(buffer);

		if (buffer_length > 1)
		{
			Glib::ustring  message ( _("Sorry, but you cannot paste over existing chord.\n\
 First create a space for it using New Chord button.") );
			
			Gtk::MessageDialog dialog(*this,  message );
			dialog.run();
			return;
		}
		
		// copy all notes to new location
		for(int i = 1; i<length; i++)
		{
			buffer_note = (Note *) g_list_nth_data(copied_chord, i);
			Note * n = new Note(buffer_note->midi_number);
			n->normal_x = (selected_chord_number - 1) * 53 ;
			n->update_x();
			n->velocity = buffer_note->velocity;
			n->midi_channel = buffer_note->midi_channel;
			n->midi_instrument = buffer_note->midi_instrument;
			n->display = buffer_note->display;
			n->update_display();
			area.current_chord = g_list_append(area.current_chord, n );
		}
		area.queue_draw();
	}	
}

// This function is called two times in the code, one from close_and_save(), 
// and one from clicking save button in the main window
// This function shows window with entry box where new name can be entered,
// saves current sequence along with other sequences
// into a file saved_sequences.txt
void ExampleWindow::on_save()
{
	GList * buffer_chord_link, * buffer_chord;
	int chord_length, sequence_length;
    bool is_empty = true;
    sequence_length = g_list_length(area.current_sequence);
    
    // check if current sequence is empty (not counting dummy notes)
    for(int i = 0; i < sequence_length; i++)
	{
		buffer_chord_link = (GList *) g_list_nth(area.current_sequence, i);
		buffer_chord = (GList *) buffer_chord_link->data; //chord
			
	//	std::cout << "i is " << i << std::endl;
		
		chord_length = g_list_length(buffer_chord);
		//std::cout << "Chord length is " << chord_length << std::endl;
		if(chord_length > 1)
		{
			is_empty = false;
			break;
		}
	}
	
	if(is_empty)
	{
		Glib::ustring  message ( _("Nothing to save") );
		
		Gtk::MessageDialog dialog(*this, message);
		dialog.run();
		return;
	}
	
	// if sequence is saved, i.e. has a name, display a dialog that will
	// give a choice whether to rename the current sequence or not
	if(is_sequence_named)
	{
		rename_dialog = new Gtk::Window;
		Gtk::Box *r_vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0) );
		rename_dialog->add(*r_vbox);
		Gtk::Grid *r_grid = Gtk::manage(new Gtk::Grid);
		r_grid->set_border_width(10);
		r_grid->set_row_spacing(5);
		r_vbox->add(*r_grid);
		Gtk::Label *r_lb = Gtk::manage(new Gtk::Label(_("Do you want to rename or save current sequence?") ) );
		r_grid->attach(*r_lb, 0, 0, 3, 1);
		
		Gtk::Button * button_rename = Gtk::manage(new Gtk::Button(_("Rename") ) );
		button_rename->signal_clicked().connect(sigc::bind<Gtk::Window*>(sigc::mem_fun 
				(*this, &ExampleWindow::rename_current_sequence), rename_dialog) );
		r_grid->attach(*button_rename, 0, 1, 1, 1);
		
		Gtk::Button * button_justsave = Gtk::manage(new Gtk::Button(_("Just Save") ) );
		button_justsave->signal_clicked().connect(sigc::bind<Gtk::Window*>(sigc::mem_fun 
				(*this, &ExampleWindow::just_save_current_sequence), rename_dialog) );
		r_grid->attach(*button_justsave, 1, 1, 1, 1);	
		
		Gtk::Button * button_cancel = Gtk::manage(new Gtk::Button(_("Cancel") ) );	
		button_cancel->signal_clicked().connect(sigc::bind<Gtk::Window*>(sigc::mem_fun 
				(*this, &ExampleWindow::cancel_rename_current_sequence), rename_dialog) );
		r_grid->attach(*button_cancel, 2, 1, 1, 1);
		r_vbox->show_all();
		rename_dialog->show();		
	}
	else
	{
	// if current sequence is not saved yet, pop up a dialog that asks
	// for a name

	save_dialog = new Gtk::Window;
	Gtk::Box *vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0) );
    save_dialog->add(*vbox);
	Gtk::Grid *grid = Gtk::manage(new Gtk::Grid);
    grid->set_border_width(10);
    grid->set_row_spacing(5);
    vbox->add(*grid);
	Gtk::Label *lb = Gtk::manage(new Gtk::Label(_("Please enter descriptive name for this sequence:") ));
    grid->attach(*lb, 0, 0, 1, 1);
	Gtk::Entry *name = Gtk::manage(new Gtk::Entry() );
    name->set_hexpand(true);
	grid->attach(*name, 0, 1, 1, 1);
	
	Gtk::Button *blogin = Gtk::manage(new Gtk::Button(_("Save Sequence") ) );
    blogin->signal_clicked().connect(sigc::bind<Gtk::Entry*>(sigc::mem_fun 
				(*this, &ExampleWindow::save_current_sequence), name) );
    grid->attach(*blogin, 0, 2, 1, 1);

    vbox->show_all();
	save_dialog->show();
	}

}

void ExampleWindow::save_current_sequence(Gtk::Entry *name)
{
	Glib::ustring * buffer, * second_buffer;
	buffer = new Glib::ustring(name->get_text() );

	if(buffer->length() == 0) // empty string
	{
		Glib::ustring message(_("Empty string is not allowed") );
		
		Gtk::MessageDialog dialog(*this, message);
		dialog.run();
		return;
	}

	// check if name already exist in the list of other names
	for(int i=0; i<g_list_length(sequence_names); i++)
	{
		second_buffer = (Glib::ustring *) g_list_nth_data(sequence_names, i);
		if (*second_buffer == *buffer)
		{
			Glib::ustring message(_("This name is already in the list") );
			
			Gtk::MessageDialog dialog(*this, message);
			dialog.run();
			return;
		}
	} 
	// if we got here, that means name is not on the list: add it to the list

	// if sequence is named,  change the name
	if(is_sequence_named)
	{ 
		GList * link = g_list_find(sequence_names, current_sequence_name);
		delete current_sequence_name;
		current_sequence_name = buffer;
		
		link->data = current_sequence_name;
		Glib::ustring title = "Chord Sequencer : " + *buffer;
		set_title(title);
	}	
	else // (! is_sequence_named)
	{
		// new unnamed sequence is always at index 0
		sequence_names = g_list_prepend(sequence_names, buffer ) ;
		current_sequence_name = buffer;
		Glib::ustring title = "Chord Sequencer : " + *buffer;
		set_title(title);
	}	
	is_sequence_named = true;
	
	// Save all sequences into a file:
	save_all();
	save_dialog->hide();
}

// This function saves all sequences into a file saved_sequences.txt
void ExampleWindow::save_all()
{
	Glib::ustring * buffer;
	int total_length;
	std::ofstream out("/usr/local/var/chordsequencer/saved_sequences.txt");
	if(!out)
	{
		std::cout << _("Cannot open file /usr/local/var/chordsequencer/saved_sequences.txt") << std::endl;
		return;
	}
	
	//std::cout << "Inside save_all()\n";
	if(is_sequence_named)
		total_length = g_list_length(area.sequences);
	else
		total_length = g_list_length(area.sequences) - 1;
	int sequence_length, chord_length;
	GList * buffer_sequence, * buffer_chord;
	Note * buffer_note;
	Glib::ustring * buffer_name;
	
	for(int i=0; i<total_length; i++)
	{
		if(is_sequence_named)
			buffer_sequence = (GList *) (g_list_nth(area.sequences, i)->data);
		else
			buffer_sequence = (GList *) (g_list_nth(area.sequences, i + 1)->data);
			
		buffer_name = (Glib::ustring *) (g_list_nth(sequence_names, i)->data);
		sequence_length = g_list_length(buffer_sequence);
		out << "Sequence <" << *buffer_name << ">, size=" << sequence_length << std::endl;
	
		for(uint j=0; j < sequence_length; j++)
		{
			buffer_chord = (GList *) (g_list_nth(buffer_sequence, j)->data);
			chord_length = g_list_length(buffer_chord);
			out << "\tChord <" << (j+1) << ">" << std::endl;
	
			for(uint k=1; k < chord_length; k++)
			{
				buffer_note = (Note *) (g_list_nth(buffer_chord, k)->data);
				// write down all 6 attributes of the note, 
				// skipping buffer notes
				out << "\t\tNote <" << buffer_note->midi_number << ">" << std::endl;
				out << "\t\t\tvelocity = " << buffer_note->velocity << std::endl;
				out << "\t\t\tmidi_instrument = " << buffer_note->midi_instrument << std::endl;
				out << "\t\t\tdisplay_options = " << buffer_note->display_options << std::endl;
				out << "\t\t\tdisplay = " << buffer_note->display << std::endl;
				out << "\t\t\tmidi_channel = " << buffer_note->midi_channel << std::endl;
			}
		}
	} 
	out.close();
	//puts("saved");
}

void ExampleWindow::rename_current_sequence(Gtk::Window* win)
{
	save_dialog = new Gtk::Window;
	Gtk::Box *vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0) );
    save_dialog->add(*vbox);
	Gtk::Grid *grid = Gtk::manage(new Gtk::Grid);
    grid->set_border_width(10);
    grid->set_row_spacing(5);
    vbox->add(*grid);
	Gtk::Label *lb = Gtk::manage(new Gtk::Label(_("Please enter descriptive name for this sequence:")));
    grid->attach(*lb, 0, 0, 1, 1);
	Gtk::Entry *name = Gtk::manage(new Gtk::Entry() );
    name->set_hexpand(true);
	grid->attach(*name, 0, 1, 1, 1);
	
	Gtk::Button *blogin = Gtk::manage(new Gtk::Button(_("Save Sequence") ) );
    blogin->signal_clicked().connect(sigc::bind<Gtk::Entry*>(sigc::mem_fun 
				(*this, &ExampleWindow::save_current_sequence), name) );
    grid->attach(*blogin, 0, 2, 1, 1);

    vbox->show_all();
	save_dialog->show();
	win->hide();
}

// This function is called when user clicks save button and then chooses
// "Just Save" option from the popup dialog
void ExampleWindow::just_save_current_sequence(Gtk::Window* win)
{
	win->hide();
	save_all();
}

// This function is called when user clicks save button and then chooses
// "Cancel" option from the popup dialog
void ExampleWindow::cancel_rename_current_sequence(Gtk::Window* win)
{
	puts(_("canceling...") );
	win->hide();
}



ExampleWindow::SequenceWindow::SequenceWindow()
: m_Button_Select(_("Select") ),
m_Button_Delete(_("Delete") ), 
m_Button_New(_("New Sequence") ), 
m_Button_Cancel(_("Cancel") )  
{
	set_title(_("Sequences") );
	set_border_width(5);
	set_default_size(400, 200);
	add(m_VBox);
	//Add the TreeView, inside a ScrolledWindow, with the button underneath:
	m_ScrolledWindow.add(m_TreeView);
	//Only show the scrollbars when they are necessary:
	m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	m_VBox.pack_start(m_ScrolledWindow);
	m_VBox.pack_start(m_ButtonBox, Gtk::PACK_SHRINK);
	m_ButtonBox.pack_start(m_Button_New, Gtk::PACK_SHRINK);
	m_ButtonBox.pack_start(m_Button_Delete, Gtk::PACK_SHRINK);
	m_ButtonBox.pack_start(m_Button_Select, Gtk::PACK_SHRINK);
	m_ButtonBox.pack_start(m_Button_Cancel, Gtk::PACK_SHRINK);
	m_ButtonBox.set_border_width(5);
	m_ButtonBox.set_layout(Gtk::BUTTONBOX_START);
	
	//Create the Tree model:
	m_refTreeModel = Gtk::ListStore::create(m_Columns);
	m_TreeView.set_model(m_refTreeModel);

	//Add the TreeView’s view columns:
	//This number will be shown with the default numeric formatting.
	m_TreeView.append_column("ID", m_Columns.m_col_id);
	m_TreeView.append_column("Name", m_Columns.m_col_name);
	
}

ExampleWindow::SequenceWindow::~SequenceWindow()
{
}

void ExampleWindow::SequenceWindow::send_data(GList * s_names)
{
	m_refTreeModel->clear();
	//Fill the TreeView’s model
	Gtk::TreeModel::Row row;

	// iterate through the sequence names and display each
	
	for(int i=0; i<g_list_length(s_names); i++)
	{
		Glib::ustring * name = (Glib::ustring *)g_list_nth_data( 
													s_names, i);
		row = *(m_refTreeModel->append());
		row[m_Columns.m_col_id] = i;
		row[m_Columns.m_col_name] = *name;
	}

	Glib::RefPtr<Gtk::TreeSelection> selection = m_TreeView.get_selection();
	selection->set_mode(Gtk::SELECTION_BROWSE);

	show_all_children();
}
	

// This function is called when m_Button_Select from SequenceWindow is clicked
void ExampleWindow::select_sequence(Gtk::TreeView * view)
{
	GList * buffer_chord_link, * buffer_note_link, * buffer_chord;
	int chord_length, sequence_length;
    bool is_empty = true;
    sequence_length = g_list_length(area.current_sequence);
	
	Glib::RefPtr<Gtk::TreeSelection> selection = view->get_selection();
	Gtk::TreeModel::iterator selectedRow = selection->get_selected();
	Gtk::TreeModel::Row m_row = *selectedRow;
	Glib::ustring str = m_row.get_value(m_Columns.m_col_name);
	//std::cout << "inside select seq " << str << std::endl;
	
	
	// check if current sequence is empty (not counting dummy notes)
    for(int i = 0; i < sequence_length; i++)
	{
		buffer_chord_link = (GList *) g_list_nth(area.current_sequence, i);
		buffer_chord = (GList *) buffer_chord_link->data; //chord
			
	//	std::cout << "i is " << i << std::endl;
		
		chord_length = g_list_length(buffer_chord);
		//std::cout << "Chord length is " << chord_length << std::endl;
		if(chord_length > 1)
		{
			is_empty = false;
			break;
		}
	}
	
	if(is_empty)
	{
		close_and_select(NULL, str);
		return;
	}
	
	// if current sequence does not have a name, ask for a name to save it
	if(! is_sequence_named)
	{
		save_or_not_dialog = new Gtk::Window;
		Gtk::Box *sn_vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0) );
		save_or_not_dialog->add(*sn_vbox);
		Gtk::Grid *sn_grid = Gtk::manage(new Gtk::Grid);
		sn_grid->set_border_width(10);
		sn_grid->set_row_spacing(5);
		sn_vbox->add(*sn_grid);
		Gtk::Label * sn_lb = Gtk::manage(new Gtk::Label(_("Do you want to save the current sequence?")));
		sn_grid->attach(*sn_lb, 0, 0, 2, 1);
		
		Gtk::Button * sn_button_rename = Gtk::manage(new Gtk::Button(_("Save") ) );
		sn_button_rename->signal_clicked().connect(sigc::bind<Gtk::Window*>(sigc::mem_fun 
				(*this, &ExampleWindow::close_and_save), save_or_not_dialog) );
		sn_grid->attach(*sn_button_rename, 0, 1, 1, 1);
		
		// multiple arguments binding example !
		Gtk::Button * sn_button_cancel = Gtk::manage(new Gtk::Button(_("Don't save") ) );
		sn_button_cancel->signal_clicked().connect(sigc::bind<Gtk::Window*, Glib::ustring>(sigc::mem_fun 
				(*this, &ExampleWindow::close_and_select), save_or_not_dialog, str) );
		sn_grid->attach(*sn_button_cancel, 1, 1, 1, 1);		
		sn_vbox->show_all();
		save_or_not_dialog->show();		
	}
	else // current sequence has a name
	{
		// make selected sequence current - display on the screen
		// find the number of selected sequence
		// At this point the length of sequence_names equals length of area.sequences
		Glib::ustring * buf_str;
		int number;
		for(int i=0; i<g_list_length(sequence_names);i++)
		{
			buf_str = (Glib::ustring *) g_list_nth_data(sequence_names, i);
			if(*buf_str == str)
			{
				number = i;
				break;
			}
		}
		
		// make selected sequence current
		area.current_sequence = (GList *) g_list_nth_data(area.sequences, number);
		
		// empty selected_notes and selected_note
		for(int i=1; i<g_list_length(selected_notes); i++)
		{
			GList * buffer_list = g_list_nth(selected_notes, 1);
			Note * buffer_note = (Note *) buffer_list->data;
			buffer_note->selected = false;
			selected_notes = g_list_delete_link(selected_notes, buffer_list);
		}
		if(selected_note != NULL)
			selected_note->selected = false;
		// selected_note = NULL;
		set_velocity_b.set_sensitive(false);
		sc->set_sensitive(false);
		
		
		// change the title
		current_sequence_name = buf_str;
		Glib::ustring title = "Chord Sequencer : " + *current_sequence_name;
		set_title(title);
		is_sequence_named = true;
			// change selected_chord_number and selected_chord
		area.current_chord = (GList *) g_list_nth_data(area.current_sequence, 0);
		selected_chord_number = 1;
		
		area.queue_draw();
		
		int seq_length = g_list_length(area.sequences);
		int str_length = g_list_length(sequence_names);
	//	std::cout << "Sequences length: " << seq_length << ", sequence_names length: "<< str_length << std::endl;
		
		// mark corresponding chord buttons - 0 start notation
		int cur_seq_length = g_list_length(area.current_sequence);
		for (int i=0; i<64; i++)
		{
			if(i<cur_seq_length)
				chord_radio_buttons[i]->occupied = true;
			else
				chord_radio_buttons[i]->occupied = false;
			chord_radio_buttons[i]->pressed = false;
			chord_radio_buttons[i]->queue_draw (); 
		}
		chord_radio_buttons[0]->pressed = true;
		chord_radio_buttons[0]->queue_draw();
	//	std::cout << "Current sequence length: " << cur_seq_length << std::endl;
	}
	// close sequences window
	s_win.hide();
}

// This function is called when the user clicks on the "Delete" button 
// in the sequence window
void ExampleWindow::delete_sequence(Gtk::TreeView * view)
{
	Glib::RefPtr<Gtk::TreeSelection> selection = view->get_selection();
	Gtk::TreeModel::iterator selectedRow = selection->get_selected();
	Gtk::TreeModel::Row m_row = *selectedRow;
	Glib::ustring str = m_row.get_value(m_Columns.m_col_name);
	//std::cout << "inside delete seq " << str << std::endl;
	
	// ask are you sure want to delete?
	areusure_dialog = new Gtk::Window;
	Gtk::Box * areusure_vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0) );
	areusure_dialog->add(*areusure_vbox);
	Gtk::Grid *areusure_grid = Gtk::manage(new Gtk::Grid);
	areusure_grid->set_border_width(10);
	areusure_grid->set_row_spacing(5);
	areusure_vbox->add(*areusure_grid);
	Gtk::Label *areusure_lb = Gtk::manage(new Gtk::Label(_("Are you sure you want to delete this sequence?")));
	areusure_grid->attach(*areusure_lb, 0, 0, 2, 1);
	
	Gtk::Button * button_yes = Gtk::manage(new Gtk::Button(_("Delete") ) );
	button_yes->signal_clicked().connect(sigc::bind<Gtk::Window*, Glib::ustring>(sigc::mem_fun 
			(*this, &ExampleWindow::delete_sequence_yes), areusure_dialog, str) );
	areusure_grid->attach(*button_yes, 0, 1, 1, 1);
	
	Gtk::Button * button_no = Gtk::manage(new Gtk::Button(_("Cancel") ) );
	button_no->signal_clicked().connect(sigc::bind<Gtk::Window*>(sigc::mem_fun 
			(*this, &ExampleWindow::cancel_delete_sequence), areusure_dialog) );
	areusure_grid->attach(*button_no, 1, 1, 1, 1);		
	areusure_vbox->show_all();
	areusure_dialog->show();		
}

// This function is called when "New sequence" button is pressed in 
// sequence window
void ExampleWindow::new_sequence()
{
	
	GList * buffer_chord_link, * buffer_note_link, * buffer_chord;
	int chord_length, sequence_length;
    bool is_empty = true;
    sequence_length = g_list_length(area.current_sequence);
    
    // check if current sequence is empty (not counting dummy notes)
    for(int i = 0; i < sequence_length; i++)
	{
		buffer_chord_link = (GList *) g_list_nth(area.current_sequence, i);
		buffer_chord = (GList *) buffer_chord_link->data; //chord
			
	//	std::cout << "i is " << i << std::endl;
		
		chord_length = g_list_length(buffer_chord);
		//std::cout << "Chord length is " << chord_length << std::endl;
		if(chord_length > 1)
		{
			is_empty = false;
			break;
		}
	}
	
	if(! is_sequence_named && is_empty)
	{
		// close sequences window
		area.queue_draw();
		s_win.hide();
		return;
	}
	// if current sequence does not have a name, ask for a name to save it
	else if(! is_sequence_named && ! is_empty )
	{
		save_or_not_dialog = new Gtk::Window;
		Gtk::Box *sn_vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0) );
		save_or_not_dialog->add(*sn_vbox);
		Gtk::Grid *sn_grid = Gtk::manage(new Gtk::Grid);
		sn_grid->set_border_width(10);
		sn_grid->set_row_spacing(5);
		sn_vbox->add(*sn_grid);
		Gtk::Label * sn_lb = Gtk::manage(new Gtk::Label(_("Do you want to save the current sequence?")));
		sn_grid->attach(*sn_lb, 0, 0, 2, 1);
		
		Gtk::Button * sn_button_rename = Gtk::manage(new Gtk::Button(_("Save") ) );
		sn_button_rename->signal_clicked().connect(sigc::bind<Gtk::Window*>(sigc::mem_fun 
				(*this, &ExampleWindow::close_and_save), save_or_not_dialog) );
		sn_grid->attach(*sn_button_rename, 0, 1, 1, 1);
		
		Gtk::Button * sn_button_cancel = Gtk::manage(new Gtk::Button(_("Don't save") ) );
		sn_button_cancel->signal_clicked().connect(sigc::bind<Gtk::Window*>(sigc::mem_fun 
				(*this, &ExampleWindow::close_and_new), save_or_not_dialog) );
		sn_grid->attach(*sn_button_cancel, 1, 1, 1, 1);		
		sn_vbox->show_all();
		save_or_not_dialog->show();		
		return;
	}
	
	// make a new sequence 
	Note * dummy_note = new Note(-1000);
	GList * new_seq = new GList();
	GList * new_chord = new GList();
	new_chord->data = dummy_note;
	new_seq->data = new_chord;
	
	// add it to the list - new unnamed sequence is always at index 0
	area.sequences = g_list_prepend(area.sequences, new_seq);
	// make it current sequence - show on the screen
	area.current_sequence = new_seq;
	set_title("Chord Sequencer");
	// Do not change current sequence name - !!! --
	// this is done in save_current_sequence() when the proper button is
	// clicked
		
	// change selected chord and selected chord number
	selected_chord_number = 1;
	area.current_chord = new_chord;
	is_sequence_named = false;
	// close sequences window
	area.queue_draw();
	s_win.hide();
	
}

// This function is called when the user selected another sequence or 
// clicked new sequence button and decided to save the current one. 
// The current sequence does not have a name yet.
void ExampleWindow::close_and_save(Gtk::Window* win)
{
	// This function will be called alone when the user clicks save button
	// in the main window
	on_save();
	
	win->hide();
	s_win.hide();
}


// This function is called when the user selected another sequence and
// decided not to save the current one, or the current sequence is empty
// (in that case win is NULL) The current sequence does not 
// have a name yet.
void ExampleWindow::close_and_select(Gtk::Window* win, Glib::ustring str)
{
	//std::cout << "Str is " << str << std::endl;
	if(str.length() == 0)
	{
		Gtk::MessageDialog dialog(*this, _("Nothing to select") );
		dialog.run();
	}
	else
	{	
		// if we got here, that means selected sequence has a name, 
		// but current_sequence does not have a name.
		GList * buffer_chord_link, * buffer_note_link, * buffer_chord;
		GList * current_sequence_node = g_list_find (area.sequences, area.current_sequence);
        GList * current_name_node = g_list_find (sequence_names,
             current_sequence_name);
        Note * buffer_note;
        int chord_length, sequence_length;
    
        	// empty selected_notes and selected_note (leave dummy_selected_note)
		for(int i=1; i<g_list_length(selected_notes); i++)
		{
			GList * buffer_list = g_list_nth(selected_notes, 1);
			Note * buffer_note = (Note *) buffer_list->data;
			buffer_note->selected = false;
			selected_notes = g_list_delete_link(selected_notes, buffer_list);
		}
		if(selected_note != NULL)
			selected_note->selected = false;
		// selected_note = NULL;
		set_velocity_b.set_sensitive(false);
		sc->set_sensitive(false);
		
		// delete current sequence node and everything inside
		sequence_length = g_list_length(area.current_sequence);
		//std::cout << "Sequence length is " << sequence_length << std::endl;
		for(int i = 0; i < sequence_length; i++)
		{
			buffer_chord_link = (GList *) g_list_last(area.current_sequence);
			buffer_chord = (GList *) buffer_chord_link->data; //chord
			
	//		std::cout << "i is " << i << std::endl;
			// delete all notes
			chord_length = g_list_length(buffer_chord);
			//std::cout << "Chord length is " << chord_length << std::endl;
			for(int j = 0; j<chord_length; j++)
			{
				buffer_note_link = (GList *) g_list_last(buffer_chord);
	//??		buffer_note = (Note *) buffer_note_link->data;
	//??		delete buffer_note;
	//			std::cout << "j is " << j << std::endl;
				// delete buffer_note node, except for the first one (head)
				if (j < (chord_length - 1) )
				{
					buffer_chord = g_list_delete_link (buffer_chord,  buffer_note_link);
				}
			}
			// delete buffer_chord node, except for the first one (head)
			if(i < (sequence_length - 1) )
				area.current_sequence = g_list_delete_link (area.current_sequence, buffer_chord_link);
		}
		
		// delete current sequence node
		area.sequences = g_list_delete_link (area.sequences, current_sequence_node);
		
		// Sequence_names does not need to be changed
		
		// find the number of selected sequence
		// At this point the length of sequence_names equals length of area.sequences
		Glib::ustring * buf_str;
		int number;
		for(int i=0; i<g_list_length(sequence_names);i++)
		{
			buf_str = (Glib::ustring *) g_list_nth_data(sequence_names, i);
			if(*buf_str == str)
			{
				number = i;
				break;
			}
		}
		
		// make selected sequence current
		area.current_sequence = (GList *) g_list_nth_data(area.sequences, number);
		// change the title
		current_sequence_name = buf_str;
		Glib::ustring title = "Chord Sequencer : " + *current_sequence_name;
		set_title(title);
		is_sequence_named = true;
		// change selected chord
		area.current_chord = (GList *) g_list_nth_data(area.current_sequence, 0);
		selected_chord_number = 1;
		
		area.queue_draw();
		
		int seq_length = g_list_length(area.sequences);
		int str_length = g_list_length(sequence_names);
	//	std::cout << "Sequences length: " << seq_length << ", sequence_names length: "<< str_length << std::endl;
		
		// mark corresponding chord buttons - 0 start notation
		int cur_seq_length = g_list_length(area.current_sequence);
		for (int i=0; i<64; i++)
		{
			if(i<cur_seq_length)
				chord_radio_buttons[i]->occupied = true;
			else
				chord_radio_buttons[i]->occupied = false;
			chord_radio_buttons[i]->pressed = false;
			chord_radio_buttons[i]->queue_draw (); 
		}
		chord_radio_buttons[0]->pressed = true;
		chord_radio_buttons[0]->queue_draw();
	//	std::cout << "Current sequence length: " << cur_seq_length << std::endl;

	}
	if(win != NULL)
		win->hide();
	s_win.hide();
}

// This function is called when the user pushed new sequence button and
// decided not to save the current one. The current sequence does not 
// have a name yet.
void ExampleWindow::close_and_new(Gtk::Window* win)
{
	
	GList * buffer_chord_link, * buffer_note_link, * buffer_chord;
	GList * current_sequence_node = g_list_find (area.sequences, area.current_sequence);
    Note * buffer_note;
    int chord_length, sequence_length;
    
       	// empty selected_notes and selected_note (leave dummy_selected_note)
	for(int i=1; i<g_list_length(selected_notes); i++)
	{
		GList * buffer_list = g_list_nth(selected_notes, 1);
		Note * buffer_note = (Note *) buffer_list->data;
		buffer_note->selected = false;
		selected_notes = g_list_delete_link(selected_notes, buffer_list);
	}
	if(selected_note != NULL)
		selected_note->selected = false;
	// selected_note = NULL;
	set_velocity_b.set_sensitive(false);
	sc->set_sensitive(false);
		
	// delete everything inside current sequence node
	sequence_length = g_list_length(area.current_sequence);
	//std::cout << "Sequence length is " << sequence_length << std::endl;
	for(int i = 0; i < sequence_length; i++)
	{
		buffer_chord_link = (GList *) g_list_last(area.current_sequence);
		buffer_chord = (GList *) buffer_chord_link->data; //chord
			
	//	std::cout << "i is " << i << std::endl;
		// delete all notes
		chord_length = g_list_length(buffer_chord);
		//std::cout << "Chord length is " << chord_length << std::endl;
		//std::cout << "i is " << i << std::endl;
		for(int j = 0; j<chord_length; j++)
		{
			buffer_note_link = (GList *) g_list_last(buffer_chord);
	//??	buffer_note = (Note *) buffer_note_link->data;
	//??	delete buffer_note;
			//std::cout << "j is " << j << std::endl;
			// delete buffer_note node, except for the first one(dummy) 
			if (j < (chord_length - 1) )
			{
				buffer_chord = g_list_delete_link (buffer_chord,  buffer_note_link);
				//std::cout << "removing note  " << j << std::endl;
			}
		}
		// delete buffer_chord node, except for the first one
		if(i < (sequence_length - 1) )
			area.current_sequence = g_list_delete_link (area.current_sequence, buffer_chord_link);
	}
	// change selected chord
	area.current_chord = (GList *) g_list_nth_data(area.current_sequence, 0);
	selected_chord_number = 1;
		
	area.queue_draw();
	// mark corresponding chord buttons - 0 start notation
	int cur_seq_length = g_list_length(area.current_sequence);
	for (int i=0; i<64; i++)
	{
		if(i<cur_seq_length)
			chord_radio_buttons[i]->occupied = true;
		else
			chord_radio_buttons[i]->occupied = false;
		chord_radio_buttons[i]->pressed = false;
		chord_radio_buttons[i]->queue_draw (); 
	}
	chord_radio_buttons[0]->pressed = true;
	chord_radio_buttons[0]->queue_draw();	
	
	win->hide();
	s_win.hide();
	
} // end close_and_new()

void ExampleWindow::cancel_delete_sequence(Gtk::Window* w)
{
	puts(_("canceling...") );
	w->hide();
}

// This function is called after the user confirms deleting sequence in the 
// sequence window
void ExampleWindow::delete_sequence_yes(Gtk::Window* wind, Glib::ustring stri)
{
	// find the number of the sequence to delete
	Glib::ustring * buf_str;
	int sequence_number, sequence_name_number;
	
	int length_names, length_seqs;
	length_names = g_list_length(sequence_names);
	length_seqs = g_list_length(area.sequences);
	// std::cout << "Sequences length: " << length_seqs << ", names length: " << length_names << std::endl;
	
	for(int i=0; i<g_list_length(sequence_names);i++)
	{
		buf_str = (Glib::ustring *) g_list_nth_data(sequence_names, i);
		if(*buf_str == stri)
		{
			if(is_sequence_named)
			{
				sequence_number = i;
				sequence_name_number = i;
			}
			else
			{	// unnamed sequence is always at index 0
				sequence_number = i + 1;
				sequence_name_number = i;
			}
			break;
		}
	}
	GList * sequence_node_to_delete = g_list_nth(area.sequences, sequence_number);
	GList * sequence_data_to_delete = (GList *) sequence_node_to_delete->data;
	
	if(  sequence_data_to_delete == area.current_sequence )
	{
		Glib::ustring message(_("Cannot delete current sequence,\n\
 select another sequence first") );
		
		Gtk::MessageDialog dialog(*this, message);
		dialog.run();
		wind->hide();
		return;
	}
		// delete all notes and chords from the sequence to delete
		// delete current sequence node and everything inside
	int sequence_length = g_list_length(sequence_data_to_delete);
	int chord_length;
	GList * buffer_chord_link, * buffer_chord, * buffer_note_link; 
	Note * buffer_note;
	
	//std::cout << "Number of sequence to delete is " << sequence_number << std::endl;
	//std::cout << "Number of sequence name to delete is " << sequence_name_number << std::endl;
	//std::cout << "Sequence length is " << sequence_length << std::endl;
	
	
	for(int i = 0; i < sequence_length; i++)
	{
		buffer_chord_link = (GList *) g_list_last(sequence_data_to_delete);
		buffer_chord = (GList *) buffer_chord_link->data; //chord
		
	//	std::cout << "i is " << i << std::endl;
		// delete all notes
		chord_length = g_list_length(buffer_chord);
	//	std::cout << "Chord length is " << chord_length << std::endl;
		for(int j = 0; j<chord_length; j++)
		{
			buffer_note_link = (GList *) g_list_last(buffer_chord);
			buffer_note = (Note *) buffer_note_link->data;
			//delete buffer_note;
	//		std::cout << "j is " << j << std::endl;
			// delete buffer_note node (except for the head node)
			if(j < chord_length -1)
				buffer_chord = g_list_delete_link (buffer_chord,  buffer_note_link);
		}
		// delete buffer_chord node (except for the head node)
		if(i < sequence_length - 1)
			sequence_data_to_delete = g_list_delete_link (sequence_data_to_delete, buffer_chord_link);
	}
	// delete corresponding name in the name list
	GList * string_node_to_delete = g_list_nth(sequence_names, sequence_name_number);
	delete (Glib::ustring *) string_node_to_delete->data;
	sequence_names = g_list_delete_link(sequence_names, string_node_to_delete);
	 
	// delete sequence
	area.sequences = g_list_delete_link (area.sequences, sequence_node_to_delete);
	
	
	save_all();
	// close dialog window
	wind->hide();
	s_win.hide();

}

// This function reads the file, and appends to the sequence list as many sequences
// as it finds there, current sequence remains at the index 0 unchanged
void ExampleWindow::read_saved_file()
{
	GList * sequence_head, * last_sequence_link;
	
	std::string delimiter;
	int number, pos, chord_number, count;
	std::ifstream file("/usr/local/var/chordsequencer/saved_sequences.txt");
	if(!file)
	{
		std::cout << Glib::ustring::compose(
             _("Cannot open file %1."), "/usr/local/var/chordsequencer/saved_sequences.txt") << std::endl;
		return;
	}
    std::string line, token, token_buffer, name_buffer; 
    Glib::ustring * name;
    count = 0;
    while (std::getline(file, line))
    {
		count++;
	//	std::cout << "Processing line " << count << std::endl;
		delimiter = "<";
		if ( (pos = line.find(delimiter) ) != std::string::npos)// delimiter found
		{
			token_buffer = line.substr(0, pos);
			line.erase(0, pos + delimiter.length() );
			delimiter = ">";
			pos = line.find(delimiter);
			name_buffer = line.substr(0, pos); // this is what is inside < >
			// don't trim the name!
			token = trim(token_buffer);
	//		std::cout << "Trimmed token:" << token << "end" << std::endl;
			if(token.compare("Sequence") == 0 )
			// this is a sequence line
			{
	//			puts("found sequence line");
				// create a new sequence
				GList * new_seq_link = new GList();
				area.sequences = g_list_append(area.sequences, new_seq_link );
				// create a name for the sequence
	//			std::cout << "Name is " << name_buffer << std::endl;
				// in case of sequence, < > is sequence name
				name = new Glib::ustring(name_buffer);
				sequence_names = g_list_append(sequence_names, name);
			}
			else if(token.compare("Chord") == 0 )
			// this is a chord line, every sequence must have at least 1 chord
			{
	//			puts("found chord line");
				// in case of chord, < > is chord number in this sequence
				number = std::stoi(name_buffer);
	//			std::cout << "Name is " << name_buffer << std::endl;
				// create a new chord
				GList * new_chord_link = new GList();
				
				last_sequence_link = (GList *) g_list_last(area.sequences);
				
		//		std::cout << "Number is " << number << std::endl;
				if(number == 1) // first chord
					last_sequence_link->data = new_chord_link;
				else
				{
					sequence_head = (GList *) last_sequence_link->data;
					sequence_head = g_list_append(sequence_head, new_chord_link);
				}	// add dummy note to the new chord
				Note * dummy_note = new Note(-1000);
				GList * note_link = new GList();
				new_chord_link->data = note_link;
				note_link->data = dummy_note;
				int l = g_list_length( (GList *) last_sequence_link->data);
	//			std::cout << "Length of last sequence is " << l << std::endl;
			}
			else if(token.compare("Note") == 0 )
			// this is a note line
			{
	//			puts("found note line");
				// in case of a note, < > is note midi number
				number = std::stoi(name_buffer); 
	//			std::cout << "Name is " << name_buffer << std::endl;
				// create a new note
				Note * new_note = new Note(number);
				
				last_sequence_link = (GList *) g_list_last(area.sequences);
				sequence_head =  (GList *)last_sequence_link->data;
				
				
				GList * last_chord_link = (GList *) g_list_last( sequence_head );
				GList * last_chord_head = (GList *) last_chord_link->data;
				last_chord_head = g_list_append(last_chord_head, new_note);
				chord_number = g_list_length(sequence_head);
				new_note->normal_x = (chord_number - 1) * 53 ;
				new_note->update_x();
			}
			else
			// must be some error
			{
				std::cout << _("Unknown token") << std::endl;
			}
		}
        else // must be note attribute line - delimiter not found
        {
			// Process line
			delimiter = "=";
			if ( (pos = line.find(delimiter) ) != std::string::npos)// delimiter found
			{
	//			puts("Delimiter = found");
				token_buffer = line.substr(0, pos);
				token = trim(token_buffer); // this is a name of note attribute
				line.erase(0, pos + delimiter.length() );
				
				// the rest of the line must be just a number after trimming
				name_buffer = trim(line);
			
				number = std::stoi(name_buffer); // this is a value of the attribute
				last_sequence_link = (GList *) g_list_last(area.sequences);
				sequence_head = (GList *)last_sequence_link->data;
				GList * last_chord_link = (GList *) g_list_last(sequence_head);
				GList * chord_head = (GList *) last_chord_link->data;
	//			std::cout << "Number of chords in this sequence is " << g_list_length(sequence_head);
	//			puts("here");
				GList * last_note_link = (GList *) g_list_last(chord_head);
				
				Note * last_note = (Note *) last_note_link->data;
				
				if(token.compare("velocity") == 0 )
					last_note->velocity = number;
				else if(token.compare("midi_instrument") == 0 )
					last_note->midi_instrument = number;
				else if(token.compare("display_options") == 0 )
					last_note->display_options = number;
				else if(token.compare("display") == 0 )
				{
					last_note->display = number;
					last_note->update_display();
				} 

				else if(token.compare("midi_channel") == 0 )
					last_note->midi_channel = number;
				else
					std::cout << _("Found unknown note attribute") << std::endl;
			}
			else
			{
				// delimiter not found, must be an empty line
				std::cout << _("found line without any delimiter") << std::endl;
			}
		}
    }
	file.close();
}

// trims white space on both ends and tab in front
std::string ExampleWindow::trim(const std::string& str)
{
    size_t first = str.find_first_not_of(' ');
    if (std::string::npos == first) // white space not found
    {
		size_t pos = str.find_first_not_of('\t');
		if(pos != std::string::npos) // tab found
			return str.substr(pos);
		else // tab not found
			return str;
    }
    size_t last = str.find_last_not_of(' ');
    std::string sub_string = str.substr(first, (last - first + 1));
    
    size_t pos = sub_string.find_first_not_of('\t');
    if(pos != std::string::npos)
		return sub_string.substr(pos);
	else
		return sub_string;
}



// These four functions below go with play chord mode (not radio buttons)
void ExampleWindow::on_play_chord()
{
	Note * n;
	int note_number;
	int note_velocity;
	for( int i = 1; i < g_list_length(area.current_chord); i++)
	{
		n = (Note *) g_list_nth_data(area.current_chord, i);
		note_number = n->midi_number;
		note_velocity = n->velocity;
		message[0] = 144  + channel - 1;
		message[1] = note_number;
		message[2] = note_velocity;
		midiout->sendMessage( &message );
	}
	playing = true;
	area.queue_draw();
}

void ExampleWindow::on_stop_chord()
{
	// All notes off
	message[0] = 176 + channel - 1;
	message[1] = 123;
	message[2] = 0;
	midiout->sendMessage( &message );
	playing = false;
	area.queue_draw();

}

void ExampleWindow::on_next_chord()
{
	Note * n;
	int note_number;
	int note_velocity;
	
	// All notes off
	message[0] = 176 + channel - 1;
	message[1] = 123;
	message[2] = 0;
	midiout->sendMessage( &message );
	
	int sequence_length = g_list_length(area.current_sequence);
	// adjust scroller 
	if(selected_chord_number > 10 && sequence_length > 16)
	{
		double v;
		//double upper = scroll_adj->get_upper();
		if( selected_chord_number < sequence_length)
			v = (3392 / 64 ) * (selected_chord_number - 10);
		else
			v = 0;
		scroll_adj->set_value(v);
	}
	
	// change selected chord
	if(selected_chord_number < sequence_length)
	{
		area.current_chord = (GList *) g_list_nth_data(area.current_sequence, selected_chord_number);
		selected_chord_number++;
		chord_radio_buttons[selected_chord_number-2]->pressed = false;
		chord_radio_buttons[selected_chord_number-1]->pressed = true;
		chord_radio_buttons[selected_chord_number-2]->queue_draw();
		chord_radio_buttons[selected_chord_number-1]->queue_draw();
	}
	else // reached the end of sequence, loop to the beginning
	{
		area.current_chord = (GList *) g_list_nth_data(area.current_sequence, 0);
		selected_chord_number = 1;
		chord_radio_buttons[sequence_length - 1]->pressed = false;
		chord_radio_buttons[0]->pressed = true;
		chord_radio_buttons[sequence_length - 1]->queue_draw();
		chord_radio_buttons[0]->queue_draw();
	}

	// play it
	
	for( int i = 1; i < g_list_length(area.current_chord); i++)
	{
		n = (Note *) g_list_nth_data(area.current_chord, i);
		note_number = n->midi_number;
		note_velocity = n->velocity;
		message[0] = 144 + channel - 1;
		message[1] = note_number;
		message[2] = note_velocity;
		midiout->sendMessage( &message );
	}
	playing = true;
	area.queue_draw();
}

void ExampleWindow::on_previous_chord()
{
	Note * n;
	int note_number;
	int note_velocity;
	
	// All notes off
	message[0] = 176 + channel - 1;
	message[1] = 123;
	message[2] = 0;
	midiout->sendMessage( &message );
	
	// change selected chord 
	int sequence_length = g_list_length(area.current_sequence);
	if(selected_chord_number > 1)
	{
		area.current_chord = (GList *) g_list_nth_data(area.current_sequence, selected_chord_number - 2);
		selected_chord_number--;
		chord_radio_buttons[selected_chord_number]->pressed = false;
		chord_radio_buttons[selected_chord_number-1]->pressed = true;
		chord_radio_buttons[selected_chord_number]->queue_draw();
		chord_radio_buttons[selected_chord_number-1]->queue_draw();
	}
	else // reached the beginning of sequence, loop to the end
	{
		area.current_chord = (GList *) g_list_last(area.current_sequence)->data;
		selected_chord_number = sequence_length;
		chord_radio_buttons[0]->pressed = false;
		chord_radio_buttons[sequence_length - 1]->pressed = true;
		chord_radio_buttons[0]->queue_draw();
		chord_radio_buttons[sequence_length - 1]->queue_draw();
	}
	
	
	// play it
	for( int i = 1; i < g_list_length(area.current_chord); i++)
	{
		n = (Note *) g_list_nth_data(area.current_chord, i);
		note_number = n->midi_number;
		note_velocity = n->velocity;
		message[0] = 144 + channel - 1;
		message[1] = note_number;
		message[2] = note_velocity;
		midiout->sendMessage( &message );
	}
	playing = true;
	area.queue_draw();
}



bool ExampleWindow::onKeyRelease(GdkEventKey* event)
{
	guint keyvalue =  event->keyval;
	if(ks_window.total_keys_pressed > 0)
		ks_window.total_keys_pressed--;
		
	// is key in a shortcut list?
	for(int i = 0; i < ks_window.number_of_shortcuts; i++)
	{
		if(keyvalue == ks_window.shortcut_keys[i][0] || 
				keyvalue == ks_window.shortcut_keys[i][1] ) 
		{	// key is in the shortcut list
			ks_window.shortcut_keys_pressed--;
			return true;
		}
	}
	return true;
} // en donKeyRelease()


// Binding of the pedal to the next chord button, previous chord button
// and  to stop button
bool ExampleWindow::onKeyPress(GdkEventKey* event)
{
	guint keyvalue =  event->keyval;
	/*
	gchar * label = gtk_accelerator_get_label (keyvalue, (GdkModifierType ) 0 );
	gchar * name =  gdk_keyval_name (keyvalue);
	printf("Accelerator label: %s\n", label);
	printf("Accelerator name: %s\n", name);
	*/
	
	//printf("Caught key press event keyval %u\n", keyvalue);
	ks_window.total_keys_pressed++;
	// find if the key is one of the shortcut keys
	
	for(int i = 0; i < ks_window.number_of_shortcuts; i++)
	{
		if(keyvalue == ks_window.shortcut_keys[i][0] || 
				keyvalue == ks_window.shortcut_keys[i][1] ) 
		{	// key is in the shortcut list
			ks_window.shortcut_keys_pressed++;
			// check if this command has two key shortcut or one key
			if(ks_window.shortcut_keys[i][1] == 0) // one key command
			{
					activate_command(i);
					return true;
			}
			else // key found in the two key command
			{
				if(ks_window.shortcut_keys_pressed > 1)
				// at least one shortcut key is already pressed
				{
					if(ks_window.first_key_guint != 0)
					{
						int result = ks_window.two_key_search(ks_window.first_key_guint, 
										keyvalue, ks_window.shortcut_keys);
						if(result >= 0)
							activate_command(result);
						else
							std::cout << "Error: two_key_search returned negative value"
										<< std::endl;
						return true;
					}
					else
					{
						std::cout << "Error: first_key_guint is 0" << std::endl;
						return true;
					}
				}
				else // this is first shortcut key pressed
				{
					ks_window.first_key_guint = keyvalue;
					return true;
				}
			}
		}
		else // key is not in the shortcut list
			continue;
	} // end for
	return true;
} // end onKeyPress()



ExampleWindow::KeyShortcutsWindow::KeyShortcutsWindow()
: ks_grid(),
doc_label(),
sep(),
sep1(),
command_label(_("Command:") ),
shortcut_label(_("Shortcut:") ),
play_label(_("Play Chord") ),
previous_label(_("Previous Chord") ),
stop_label(_("Stop All Sounds") ),
new_label(_("New Chord") ),
copy_label(_("Copy Chord") ),
paste_label(_("Paste Chord") ),
play_entry(),
previous_entry(),
new_entry(),
stop_entry(),
copy_entry(),
paste_entry(),
learn_play_button(_("Learn") ),
learn_previous_button(_("Learn") ),
learn_new_button(_("Learn") ),
learn_stop_button(_("Learn") ),
learn_copy_button(_("Learn") ),
learn_paste_button(_("Learn") ),
button_close(_("Close") )
{
	number_of_shortcuts = 6;
	learn_mode = false;
	shortcuts_changed = false;
	learn_index = -1;
	learn_keys_pressed = 0;
	first_learn_key_pressed = 0;
	int entry_width = 150;
	int entry_width_char = 20;
	
	set_title(_("Manage Keyboard Shortcuts") );
	set_border_width(10);
	set_default_size(300,70);
	ks_grid.set_row_spacing(5);
	ks_grid.set_column_homogeneous(false);
	add((Gtk::Widget&)ks_grid);
	doc_label_string = _("To change a shortcut, click on the corresponding \
learn button, release it, then press and release \
the new key, or combination of two new keys. \
In case of two keys, it could be any two keys.");
	doc_label.set_text(doc_label_string);
	
	total_keys_pressed = 0;
	shortcut_keys_pressed = 0;
	first_key_guint = 0;
	ks_grid.attach(doc_label, 0, 0, 3, 1);
	ks_grid.attach(command_label, 0, 1, 1, 1);
	ks_grid.attach(shortcut_label, 1, 1, 1, 1);
	ks_grid.attach(sep, 0, 2, 3, 1);
	ks_grid.attach(play_label, 0, 3, 1, 1);
	
	play_entry.set_width_chars(entry_width_char);
	play_entry.set_hexpand_set(true);
	play_entry.set_size_request(entry_width, 10);
	play_entry.set_margin_left(25);
	play_entry.set_margin_right(25);
	play_entry.set_hexpand(true);
	ks_grid.attach(play_entry, 1, 3, 1, 1);
	ks_grid.attach(learn_play_button, 2, 3, 1, 1);
	
	ks_grid.attach(previous_label, 0, 4, 1, 1);
	previous_entry.set_width_chars(entry_width_char);
	previous_entry.set_hexpand_set(true);
	previous_entry.set_size_request(entry_width, 10);
	previous_entry.set_margin_left(25);
	previous_entry.set_margin_right(25);
	previous_entry.set_hexpand(true);
	ks_grid.attach(previous_entry, 1, 4, 1, 1);
	ks_grid.attach(learn_previous_button, 2, 4, 1, 1);
	
	
	ks_grid.attach(stop_label, 0, 5, 1, 1);
	stop_entry.set_width_chars(entry_width_char);
	stop_entry.set_hexpand_set(true);
	stop_entry.set_size_request(entry_width, 10);
	stop_entry.set_margin_left(25);
	stop_entry.set_margin_right(25);
	stop_entry.set_hexpand(true);
	ks_grid.attach(stop_entry, 1, 5, 1, 1);
	ks_grid.attach(learn_stop_button, 2, 5, 1, 1);
	
	ks_grid.attach(new_label, 0, 6, 1, 1);
	new_entry.set_width_chars(entry_width_char);
	new_entry.set_hexpand_set(true);
	new_entry.set_size_request(entry_width, 10);
	new_entry.set_margin_left(25);
	new_entry.set_margin_right(25);
	new_entry.set_hexpand(true);
	ks_grid.attach(new_entry, 1, 6, 1, 1);
	ks_grid.attach(learn_new_button, 2, 6, 1, 1);
	
	ks_grid.attach(copy_label, 0, 7, 1, 1);
	copy_entry.set_width_chars(entry_width_char);
	copy_entry.set_hexpand_set(true);
	copy_entry.set_size_request(entry_width, 10);
	copy_entry.set_margin_left(25);
	copy_entry.set_margin_right(25);
	copy_entry.set_hexpand(true);
	ks_grid.attach(copy_entry, 1, 7, 1, 1);
	ks_grid.attach(learn_copy_button, 2, 7, 1, 1);
	
	ks_grid.attach(paste_label, 0, 8, 1, 1);
	paste_entry.set_width_chars(entry_width_char);
	paste_entry.set_hexpand_set(true);
	paste_entry.set_size_request(entry_width, 10);
	paste_entry.set_margin_left(25);
	paste_entry.set_margin_right(25);
	paste_entry.set_hexpand(true);
	ks_grid.attach(paste_entry, 1, 8, 1, 1);
	ks_grid.attach(learn_paste_button, 2, 8, 1, 1);
	
	ks_grid.attach(sep1, 0, 9, 3, 1);
	
	ks_grid.attach(button_close, 0, 10, 3, 1);
	ks_grid.set_halign(Gtk::ALIGN_START);
	ks_grid.show_all();
	
	entries[0] = &play_entry;
	entries[1] = &previous_entry;
	entries[2] = &stop_entry;
	entries[3] = &new_entry;
	entries[4] = &copy_entry;
	entries[5] = &paste_entry;
	
	
	// Events section
	this->signal_key_press_event().connect(
		sigc::mem_fun(*this, &KeyShortcutsWindow::onSKeyPress), false);
	this->signal_key_release_event().connect(
		sigc::mem_fun(*this, &KeyShortcutsWindow::onSKeyRelease), false);
	
	
	learn_play_button.signal_clicked().connect(sigc::bind<int>(
				sigc::mem_fun(*this, &KeyShortcutsWindow::on_learn_button_clicked), 0) );
	learn_previous_button.signal_clicked().connect(sigc::bind<int>(
				sigc::mem_fun(*this, &KeyShortcutsWindow::on_learn_button_clicked), 1) );
	learn_stop_button.signal_clicked().connect(sigc::bind<int>(
				sigc::mem_fun(*this, &KeyShortcutsWindow::on_learn_button_clicked), 2) );
	learn_new_button.signal_clicked().connect(sigc::bind<int>(
				sigc::mem_fun(*this, &KeyShortcutsWindow::on_learn_button_clicked), 3) );
	learn_copy_button.signal_clicked().connect(sigc::bind<int>(
				sigc::mem_fun(*this, &KeyShortcutsWindow::on_learn_button_clicked), 4) );
	learn_paste_button.signal_clicked().connect(sigc::bind<int>(
				sigc::mem_fun(*this, &KeyShortcutsWindow::on_learn_button_clicked), 5) );
	
	
	button_close.signal_clicked().connect( sigc::mem_fun(*this, 
							&KeyShortcutsWindow::on_close_button_clicked) );
										
	//	m_button1.signal_clicked().connect( sigc::bind<Glib::ustring>( sigc::mem_fun(*this, 
	//				&HelloWorld::on_button_clicked), "button 1") );				
} // end KeyShortcutsWindow constructor

ExampleWindow::KeyShortcutsWindow::~KeyShortcutsWindow()
{ }

// This function searches 2D array for index that has both keys,
// and returns that index, if it is found, or -1 if it is not found.
int ExampleWindow::KeyShortcutsWindow::two_key_search(guint first_key,
						guint second_key, guint  two_d_array[6][2])
{
	//puts("inside two_key_search\n");
	int result = -1;
	for(int i=0; i<number_of_shortcuts; i++)
	{
		if(two_d_array[i][1] == 0)
		{
			// single key shortcut row, proceed to the next row
			continue;
		}
		else // two key shortcut row
		{
			if(two_d_array[i][0] == first_key)
			{
				if(two_d_array[i][1] == second_key)
				{
					result = i;
					break;
				}
				else
				{
					// continue search
					continue;
				}
			}
			else
			{
				if(two_d_array[i][0] == second_key)
				{
					if(two_d_array[i][1] == first_key)
					{
						result = i;
						break;
					}
					else
						continue;
				}
				else 
					continue;
			}
		}
	}
	return result;
} // end two_key_search

// trims white space on both ends and tab in front
std::string ExampleWindow::KeyShortcutsWindow::trim(const std::string& str)
{
    size_t first = str.find_first_not_of(' ');
    if (std::string::npos == first) // white space not found
    {
		size_t pos = str.find_first_not_of('\t');
		if(pos != std::string::npos) // tab found
			return str.substr(pos);
		else // tab not found
			return str;
    }
    size_t last = str.find_last_not_of(' ');
    std::string sub_string = str.substr(first, (last - first + 1));
    
    size_t pos = sub_string.find_first_not_of('\t');
    if(pos != std::string::npos)
		return sub_string.substr(pos);
	else
		return sub_string;
}

void ExampleWindow::on_action_keyboard_shortcuts()
{
   ks_window.show();
   ks_window.raise();
}

void ExampleWindow::on_action_select_sequence()
{
   //std::cout << "Inside on_action_select_sequence()." << std::endl;
   on_sequence_clicked(NULL);
   return;
}

void ExampleWindow::about_sequencer()
{
	Glib::ustring header("	Copyright (c) 2017-2018 Marvin Eagle <trailblazer77_2000@yahoo.com> \n\n");
	
	Glib::ustring aboutinfo(_( 
	"			This is Chord Sequencer. \n\n\
\
Chord Sequencer is a program to create and interactively play \
series of chords in the live performance, controlling chord \
change by pedal (or other similar device). It uses midi and \
must be connected to an external synthesizer. \n\n\
\
Chord Sequencer is free software: you can redistribute it and/or modify \
it under the terms of the GNU General Public License as published by \
the Free Software Foundation, either version 3 of the License, or \
(at your option) any later version. \n\n\
\
Chord Sequencer is distributed in the hope that it will be useful, \
but WITHOUT ANY WARRANTY; without even the implied warranty of \
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \
GNU General Public License for more details. \n\n\
\
You should have received a copy of the GNU General Public License \
along with Chord Sequencer. \
If not, see <https://www.gnu.org/licenses/>.\n\n\
\
Author: ") );
	Glib::ustring footer("Marvin Eagle <trailblazer77_2000@yahoo.com>");
	
	Glib::ustring composed_ustring(header + aboutinfo + footer);
	
	Gtk::MessageDialog dialog(*this, composed_ustring);
	dialog.run();
	
	return;
}

void ExampleWindow::on_action_play_chord()
{
	on_chord_play_clicked(NULL);
	on_play_chord();
	return;
}

void ExampleWindow::on_action_next_chord()
{
	on_chord_play_clicked(NULL);
	on_next_chord();
	return;
}

void ExampleWindow::on_action_previous_chord()
{
	on_chord_play_clicked(NULL);
	on_previous_chord();
	return;
}

void ExampleWindow::on_action_stop_chord()
{
	on_chord_play_clicked(NULL);
	on_stop_chord();
	return;
}

void ExampleWindow::on_action_copy_chord()
{
	on_chord_edit_clicked(NULL);
	on_copy_chord();
}

void ExampleWindow::on_action_paste_chord()
{
	on_chord_edit_clicked(NULL);
	on_paste_chord();
}

void ExampleWindow::on_action_new_chord()
{
	on_chord_edit_clicked(NULL);
	on_new_chord();
}
void ExampleWindow::on_action_delete_chord()
{
	on_chord_edit_clicked(NULL);
	on_delete_chord();
}

// This function reads file saved_shortcuts.txt and parses every line to tokens.
// First token is name of the command, second is the keyvalue (integer)
// Third token is optional and is the second key keyvalue (integer).
void ExampleWindow::KeyShortcutsWindow::read_shortcuts_file()
{
	//puts("inside read_shortcuts_file\n");
	std::ifstream file("/usr/local/var/chordsequencer/saved_shortcuts.txt");
	std::string delimiter;
	std::string line, name_token, value_token, u_value_buffer, name_buffer;
	Glib::ustring u_line, u_name_token, u_value_token, u_second_value_token;
	Glib::ustring u_name_buffer;
	int shortcut_index, pos, count;
	
	if(!file)
	{
		std::cout << Glib::ustring::compose(_("Cannot open file %1"), 
												"/usr/local/var/chordsequencer/saved_shortcuts.txt");
		return;
	}
	
	count = 0;
    while (std::getline(file, line))
    {
		guint u_value_guint, u_second_value_guint;
		gchar * u_value_gchar;
		gchar * u_second_value_gchar;
		
		delimiter = "=";
		count++;
		if(count > number_of_shortcuts)
		{
			//printf("number_of_shortcuts is %d\n", number_of_shortcuts);
			break;
		}
			
		//puts("inside read_shortcuts_file while loop\n");	
		u_line = (Glib::ustring) line;
		if ( (pos = u_line.find(delimiter) ) != Glib::ustring::npos)// delimiter found
		{
			//puts("delimiter found\n");
			u_name_buffer = u_line.substr(0, pos);
			u_name_token = trim(u_name_buffer);
			u_line.erase(0, pos + delimiter.length() );
						
			// find the index of the shortcut
			for(int i=0; i<number_of_shortcuts; i++)
			{
				if(u_name_token == *shortcut_strings[i])
				{
					//puts("found shortcut name\n");
					shortcut_index = i;
				}
				
			}
			// Index found. Now check how many keys are in this shortcut
			delimiter = "+";
			if ( (pos = u_line.find(delimiter) ) != Glib::ustring::npos)// delimiter found
			{
				// we have two keys
				u_value_buffer = u_line.substr(0, pos);
				u_value_token = trim(u_value_buffer);
				u_value_guint = (guint) atoi(u_value_token.c_str() );
				u_value_gchar = gdk_keyval_name ( u_value_guint);
				Glib::ustring u_value_string(u_value_gchar);
				
				u_line.erase(0, pos + delimiter.length() );
				u_second_value_token = trim(u_line);
				u_second_value_guint = (guint) atoi(u_second_value_token.c_str() );
				u_second_value_gchar = gdk_keyval_name ( u_second_value_guint);
				Glib::ustring u_second_value_string(u_second_value_gchar);
				
				// Create a ustring by placing " + " between two ustrings
				Glib::ustring u_compound_string = u_value_string + " + " +
									u_second_value_string;

				entries[shortcut_index]->set_text(u_compound_string);
						
				shortcut_keys[shortcut_index][0] = u_value_guint;
				shortcut_keys[shortcut_index][1] = u_second_value_guint;
				
			}
			else // we have one key
			{
				u_value_token = trim(u_line);
				
				u_value_guint = (guint) atoi(u_value_token.c_str() );
				// find key name
				u_value_gchar =  gdk_keyval_name ( u_value_guint);
				//printf("Key name is %s\n", u_value_gchar);
				Glib::ustring u_value_string (u_value_gchar);
	
				entries[shortcut_index]->set_text(u_value_string);	
				shortcut_keys[shortcut_index][0] = u_value_guint;
				shortcut_keys[shortcut_index][1] = 0;
			}
	
		}
		else
		{
			std::cout << Glib::ustring::compose(
             _("Error parsing file %1"), "saved_shortcuts.txt\n");
			return;
		}
	
	} // end while(std::getline(file, line))
	for (int i = 0;i<number_of_shortcuts;i++)
	{
		//printf("Shortcut_keys[i][0] is %u\n", shortcut_keys[i][0]);
	}
	
	file.close();
}


// This function is called when a key (or two key) shortcut is pressed.
// index is the index of the command in 
// ks_window.shortcut_keys and ks_window.shortcut_strings
void ExampleWindow::activate_command(int index)
{
	if(index == 0)
	{
		if(playing)
			on_next_chord();
		else
			on_play_chord();
	}
	else if(index == 1)
		on_previous_chord();
	else if(index == 2)
		on_stop_chord();
	else if(index == 3)
		on_new_chord();
	else if(index == 4)
		on_copy_chord();
	else if(index == 5)
		on_paste_chord();
	else
		puts("Error: invalid index\n");
		
}

void ExampleWindow::KeyShortcutsWindow::on_learn_button_clicked(int data)
{
	learn_mode = true;
	learn_index = data;
	//printf("Learn button clicked with data = %d\n", data);
	
	// wait for the user to click new shortcut(s)
}


// This function implements non-ordinary key shortcut system.
// First, it is inclusive in the sense that several shortcuts can be 
// activated at the same time. Second, when two shortcut keys are assigned
// to an action, it could be any two keys, not necessarily a modifier key.
// Two keys is a maximum length of a shortcut.
bool ExampleWindow::KeyShortcutsWindow::onSKeyPress(GdkEventKey* event)
{
	
	//printf("event.is_modifier = %u\n", event->is_modifier);

	if(learn_mode == true)
	{
		guint keyvalue =  event->keyval;
		learn_keys_pressed++;
		
		if(learn_keys_pressed == 1) // only one key pressed
			first_learn_key_pressed = keyvalue;
		else if(learn_keys_pressed == 2)
		{
			// save the shortcuts (two keys)
			save_two_shortcuts(first_learn_key_pressed, keyvalue, shortcut_keys,
												learn_index);
			learn_mode = false;
			learn_keys_pressed = 0;
			learn_index = -1;
		}
		//printf("learn_keys_pressed = %d\n", learn_keys_pressed);
	}
	return true;
}

bool ExampleWindow::KeyShortcutsWindow::onSKeyRelease(GdkEventKey* event)
{
	guint keyvalue =  event->keyval;
	//puts("inside onSKeyRelease\n");
	if(learn_mode == true)
	{
		learn_mode = false;
	}
	
	if(learn_keys_pressed > 0) // save the shortcut (single key)
	{
		if(first_learn_key_pressed != keyvalue)
			puts("Error: released key is not the same as pressed key\n");
		save_shortcut(keyvalue, shortcut_keys, learn_index);
		first_learn_key_pressed = 0;
		learn_keys_pressed = 0;
		learn_index = -1;
	}
	return true;
}

void ExampleWindow::KeyShortcutsWindow::save_shortcut(guint key, 
								guint two_d_array[6][2], int index)
{
	// check if the key is already used
	for(int i= 0; i<number_of_shortcuts; i++)
	{
		if(shortcut_keys[i][1] == 0 && shortcut_keys[i][0] == key)
		{
			Glib::ustring message(_("This shortcut key is already being used") );
			Gtk::MessageDialog dialog(*this, message);
			dialog.run();
			return;
		}
	}
	// update two_d_array
	two_d_array[index][0] = key;
	two_d_array[index][1] = 0;
	
	// get string representation of this two key shortcut
	gchar * key_gchar = gdk_keyval_name (key);
	//printf("key name  is %s\n", key_gchar);
	Glib::ustring key_string(key_gchar);
	// update ks_window Gtk::Entry 
	
	entries[index]->set_text(key_string);
	shortcuts_changed = true; // used later to update labels
							// inside menus
	write_shortcuts_file();
}

void ExampleWindow::KeyShortcutsWindow::save_two_shortcuts(guint first_key, 
				guint second_key, guint two_d_array[6][2], int index)
{
	//puts("inside save two shortcuts\n");
	
	// check if the key combination is already used
	int ind = two_key_search(first_key, second_key, two_d_array);
	if(ind > -1)
	{
		Glib::ustring message(_("This two key combination is already being used") );
		Gtk::MessageDialog dialog(*this, message);
		dialog.run();
		return;
	}
	else
	{
	// update two_d_array
		two_d_array[index][0] = first_key;
		two_d_array[index][1] = second_key;
	
		// update menu label -> this can only be done after the ks_window
		// closes and control returns to on_action_keyboard_shortcuts() 
		
		// get string representation of this two key shortcut
		gchar * first_key_gchar = gdk_keyval_name (first_key);
		gchar * second_key_gchar = gdk_keyval_name (second_key);
		Glib::ustring first_key_string(first_key_gchar);
		Glib::ustring second_key_string(second_key_gchar);
					
			// Create a ustring by placing " + " between two ustrings
		Glib::ustring u_compound_string = first_key_string + " + " +
										second_key_string;
		
		// update ks_window Gtk::Entry 
		entries[index]->set_text(u_compound_string);
		shortcuts_changed = true; // used later to update labels
							// inside menus
		write_shortcuts_file();
								
	}
} // end save_two_shortcuts()

void ExampleWindow::KeyShortcutsWindow::write_shortcuts_file()
{
	Glib::ustring * buffer;
	
	std::ofstream out("/usr/local/var/chordsequencer/saved_shortcuts.txt");
	if(!out)
	{
		std::cout << Glib::ustring::compose(
             _("Cannot open file %1"), "/usr/local/var/chordsequencer/saved_shortcuts.txt.\n");
		return;
	}
	for(int i=0; i<number_of_shortcuts; i++)
	{
		out << *shortcut_strings[i] << " = ";
		out << shortcut_keys[i][0] ;
		if (shortcut_keys[i][1] != 0)
			out << " + " << shortcut_keys[i][1] << std::endl;
		else
			out << std::endl;
	}

	out.close();
	out << _("file saved") << std::endl;
}

void ExampleWindow::KeyShortcutsWindow::on_close_button_clicked()
{
	hide();
}

void ExampleWindow::update_menu_labels()
{
	//puts("inside update menu labels\n");
	const gchar * s = "\t\t\t\t\t";
	size_t total_label_length = 30;
	Glib::ustring spacer(s);
	const gchar * plus = "+";
	Glib::ustring plus_sign(plus);
	
	for(int i = 0; i<ks_window.number_of_shortcuts; i++)
	{
		//  get short string representation of this two key shortcut
		gchar * first_key_gchar = gtk_accelerator_get_label( 
					ks_window.shortcut_keys[i][0],(GdkModifierType ) 0 );
		Glib::ustring first_key_string(first_key_gchar);
		size_t first_string_size = first_key_string.length();
		//printf("Index %d: size of first string is %ld\n", i, first_string_size);
		size_t command_string_size = ks_window.shortcut_strings[i]->length();	
		//printf("Index %d: size of command string is %ld\n", i, command_string_size);
		
				
		if(ks_window.shortcut_keys[i][1] != 0) // two key shortcut
		{
			gchar * second_key_gchar = gtk_accelerator_get_label( 
					ks_window.shortcut_keys[i][1],(GdkModifierType ) 0 );
			Glib::ustring second_key_string(second_key_gchar);		
			size_t second_string_size = second_key_string.length();
			
			
			//printf("Index %d: size of second string is %ld\n", i, second_string_size);
					
					
			// Create a ustring by placing " + " between two ustrings
			Glib::ustring u_compound_string = first_key_string;
			u_compound_string.append(1, '+');
			u_compound_string.append(second_key_string);
			size_t number_of_spaces = 30 - first_string_size - second_string_size - command_string_size - 1;
			size_t number_of_tabs = 0;
			if (number_of_spaces > 6)
			{
				number_of_tabs = (number_of_spaces / 4) ;
				number_of_spaces = number_of_spaces - number_of_tabs * 4 + 1;
			
			}
			
			
			//printf("Adding %ld spaces\n", number_of_spaces);							
			// Create a menu label string by adding menu item name
			
			Glib::ustring u_label_string = * ks_window.shortcut_strings[i];
			u_label_string.append(number_of_tabs, '\t');
			u_label_string.append(number_of_spaces, ' ');
			u_label_string.append(u_compound_string);
								
								
			// change menu label
			menu_items_with_shotrcuts[i]->set_label(u_label_string);
		}
		else //one key shortcut
		{
			// Create a menu label string by adding menu item name
			size_t number_of_spaces = 30 - first_string_size - command_string_size;
			size_t number_of_tabs = 0;
			if (number_of_spaces > 6)
			{
				number_of_tabs = (number_of_spaces / 4) ;
				number_of_spaces = number_of_spaces - number_of_tabs * 4;
			
			}
			//printf("Adding %ld spaces\n", number_of_spaces);
			
			
			
			Glib::ustring u_label_string = * ks_window.shortcut_strings[i];
			u_label_string.append(number_of_tabs, '\t');
			u_label_string.append(number_of_spaces, ' ');
			u_label_string.append(first_key_string);	
			// change menu label
			menu_items_with_shotrcuts[i]->set_label(u_label_string);
		}
		puts("\n");
	}
}// end update_menu_labels();

void ExampleWindow::on_action_select_chord()
{
	Glib::ustring message = _("To select a chord, when there is more than one \
chord on the screen, just click on the purple \
chord button right below the chord you want to \
select on the chord screen");
	Gtk::MessageDialog dialog(*this, message);
	dialog.run();
	return;
}

void ExampleWindow::on_action_select_note()
{
	Glib::ustring message = _("To select a note, just click on it. \
The note should turn orange when selected");
	Gtk::MessageDialog dialog(*this, message);
	dialog.run();
	return;
}

void ExampleWindow::on_action_deselect_note()
{
	Glib::ustring message = _("To deselect a note, just click on it. \
The note should turn back to light green when deselected");
	Gtk::MessageDialog dialog(*this, message);
	dialog.run();
	return;
}

void ExampleWindow::on_action_new_note()
{
	Glib::ustring message = _("To create a new note, first select the chord, \
where you want the note to appear. \
It could be either empty chord made by pressing \
New Chord button, or a chord with some other \
notes in it. Then press on one of the keys \
on the keyboard to the left.");
	Gtk::MessageDialog dialog(*this, message);
	dialog.run();
	return;
}

void ExampleWindow::on_action_delete_note()
{
	Glib::ustring message = _("To delete an existing note, first select the chord, \
where the note is located. Then press on the \
corresponding to the note key on the keyboard \
to the left (the same key that you pressed to \
create this note).");
	Gtk::MessageDialog dialog(*this, message);
	dialog.run();
	return;
}

void ExampleWindow::show_manual()
{
	manual.show();
	manual.raise();
}

void ExampleWindow::on_channel_change(Gtk::RadioMenuItem * item, int index)
{
	if(item->get_active() )
	{
		// change channel
		channel = index + 1;
		// save it to a file
		std::ofstream out("/usr/local/var/chordsequencer/channel.txt");
		if(!out)
		{
			std::cout << Glib::ustring::compose(
             _("Cannot open file %1"), "/usr/local/var/chordsequencer/channel.txt.\n");
			return;
		}
		out << channel;
		out.close();
		//puts("saved\n");
		
	}
}

void ExampleWindow::read_channel_file()
{
	std::string line;
	std::ifstream file("/usr/local/var/chordsequencer/channel.txt");
	if(!file)
	{
		std::cout << Glib::ustring::compose(
             _("Cannot open file %1"), "/usr/local/var/chordsequencer/channel.txt.\n");
		return;
	}
	std::getline(file, line);
	channel = std::stoi(line);
	
	file.close();
	
}

